@@ -1,6 +1,6 @@
 /*
  * GQview
- * (C) 2002 John Ellis
+ * (C) 2004 John Ellis
  *
  * Author: John Ellis
  *
@@ -87,6 +87,8 @@ static gint cache_sim_write_checksum(FILE *f, CacheData *cd)
 
 static gint cache_sim_write_similarity(FILE *f, CacheData *cd)
 {
+	gint success = FALSE;
+
 	if (!f || !cd || !cd->similarity) return FALSE;
 
 	if (cd->sim && cd->sim->filled)
@@ -110,19 +112,24 @@ static gint cache_sim_write_similarity(FILE *f, CacheData *cd)
 				}
 			fwrite(buf, sizeof(buf), 1, f);
 			}
+
 		fprintf(f, "\n");
-		return TRUE;
+		success = TRUE;
 		}
-	return FALSE;
+
+	return success;
 }
 
 gint cache_sim_data_save(CacheData *cd)
 {
 	FILE *f;
+	gchar *pathl;
 
 	if (!cd || !cd->path) return FALSE;
 
-	f = fopen(cd->path, "w");
+	pathl = path_from_utf8(cd->path);
+	f = fopen(pathl, "w");
+	g_free(pathl);
 
 	if (!f) return FALSE;
 
@@ -169,11 +176,12 @@ static gint cache_sim_read_dimensions(FILE *f, char *buf, int s, CacheData *cd)
 
 	if (fseek(f, - s, SEEK_CUR) == 0)
 		{
-		char b = 'a';
+		char b;
 		char buf[1024];
 		gint p = 0;
 		gint w, h;
 
+		b = 'X';
 		while (b != '[')
 			{
 			if (fread(&b, sizeof(b), 1, f) != 1) return FALSE;
@@ -211,10 +219,11 @@ static gint cache_sim_read_checksum(FILE *f, char *buf, int s, CacheData *cd)
 
 	if (fseek(f, - s, SEEK_CUR) == 0)
 		{
-		char b = 'a';
+		char b;
 		char buf[1024];
 		gint p = 0;
 
+		b = 'X';
 		while (b != '[')
 			{
 			if (fread(&b, sizeof(b), 1, f) != 1) return FALSE;
@@ -246,21 +255,34 @@ static gint cache_sim_read_similarity(FILE *f, char *buf, int s, CacheData *cd)
 {
 	if (!f || !buf || !cd) return FALSE;
 
-	if (s < 10 || strncmp("Similarity", buf, 10) != 0) return FALSE;
+	if (s < 11 || strncmp("Similarity", buf, 10) != 0) return FALSE;
+
+	if (strncmp("Grid[32 x 32]", buf + 10, 13) != 0) return FALSE;
 
 	if (fseek(f, - s, SEEK_CUR) == 0)
 		{
-		char b = 'a';
+		char b;
 		guint8 pixel_buf[3];
 		ImageSimilarityData *sd;
 		gint x, y;
 
+		b = 'X';
 		while (b != '=')
 			{
 			if (fread(&b, sizeof(b), 1, f) != 1) return FALSE;
 			}
 
-		sd = image_sim_new();
+		if (cd->sim)
+			{
+			/* use current sim that may already contain data we will not touch here */
+			sd = cd->sim;
+			cd->sim = NULL;
+			cd->similarity = FALSE;
+			}
+		else
+			{
+			sd = image_sim_new();
+			}
 
 		for (y = 0; y < 32; y++)
 			{
@@ -277,12 +299,12 @@ static gint cache_sim_read_similarity(FILE *f, char *buf, int s, CacheData *cd)
 				sd->avg_b[s + x] = pixel_buf[2];
 				}
 			}
+
 		if (fread(&b, sizeof(b), 1, f) == 1)
 			{
 			if (b != '\n') fseek(f, -1, SEEK_CUR);
 			}
 
-		image_sim_free(cd->sim);
 		cd->sim = sd;
 		cd->sim->filled = TRUE;
 		cd->similarity = TRUE;
@@ -297,12 +319,15 @@ CacheData *cache_sim_data_load(const gchar *path)
 {
 	FILE *f;
 	CacheData *cd = NULL;
-	char buf[16];
+	char buf[32];
 	gint success = TRUE;
+	gchar *pathl;
 
 	if (!path) return NULL;
 
-	f = fopen(path, "r");
+	pathl = path_from_utf8(path);
+	f = fopen(pathl, "r");
+	g_free(pathl);
 
 	if (!f) return NULL;
 
@@ -319,7 +344,7 @@ CacheData *cache_sim_data_load(const gchar *path)
 	while (success)
 		{
 		int s;
-		s = fread(&buf, sizeof(char), 16, f);
+		s = fread(&buf, sizeof(char), sizeof(buf), f);
 
 		if (s < 1)
 			{
@@ -373,22 +398,24 @@ void cache_sim_data_set_checksum(CacheData *cd, long checksum)
 
 void cache_sim_data_set_similarity(CacheData *cd, ImageSimilarityData *sd)
 {
-	ImageSimilarityData *sim;
-
 	if (!cd || !sd || !sd->filled) return;
 
-	sim = cd->sim;
-	if (!sim) sim = image_sim_new();
+	if (!cd->sim) cd->sim = image_sim_new();
 
-	memcpy(sim->avg_r, sd->avg_r, 1024);
-	memcpy(sim->avg_g, sd->avg_g, 1024);
-	memcpy(sim->avg_b, sd->avg_b, 1024);
-	sim->filled = TRUE;
+	memcpy(cd->sim->avg_r, sd->avg_r, 1024);
+	memcpy(cd->sim->avg_g, sd->avg_g, 1024);
+	memcpy(cd->sim->avg_b, sd->avg_b, 1024);
+	cd->sim->filled = TRUE;
 
-	cd->sim = sim;
 	cd->similarity = TRUE;
 }
 
+gint cache_sim_data_filled(ImageSimilarityData *sd)
+{
+	if (!sd) return FALSE;
+	return sd->filled;
+}
+
 /*
  *-------------------------------------------------------------------
  * cache path location utils
@@ -417,7 +444,7 @@ gint cache_ensure_dir_exists(gchar *path, mode_t mode)
 				if (!isdir(path))
 					{
 					if (debug) printf("creating sub dir:%s\n", path);
-					if (mkdir(path, mode) < 0)
+					if (!mkdir_utf8(path, mode))
 						{
 						printf("create dir failed: %s\n", path);
 						return FALSE;
@@ -450,7 +477,7 @@ gchar *cache_get_location(const gchar *source, gint include_name, const gchar *e
 		}
 
 	if (enable_thumb_dirs &&
-	    access(base, W_OK) == 0)
+	    access_file(base, W_OK))
 		{
 		path = g_strconcat(base, "/", GQVIEW_CACHE_DIR, name, ext, NULL);
 		if (mode) *mode = 0775;
