@@ -1,6 +1,6 @@
 /*
  * GQview
- * (C) 2002 John Ellis
+ * (C) 2004 John Ellis
  *
  * Author: John Ellis
  *
@@ -70,7 +70,7 @@ static gint collection_save_confirmed(FileDialog *fd, gint overwrite, Collection
 					collection_confirm_cancel_cb, fd);
 		g_free(buf);
 
-		generic_dialog_add(gd, _("Overwrite"), collection_confirm_ok_cb, TRUE);
+		generic_dialog_add_stock(gd, _("Overwrite"), GTK_STOCK_OK, collection_confirm_ok_cb, TRUE);
 
 		gtk_widget_show(gd->dialog);
 
@@ -158,6 +158,7 @@ static void collection_save_or_load_dialog(const gchar *path,
 	gchar *btntext;
 	void *btnfunc;
 	gchar *base;
+	const gchar *stock_id;
 
 	if (type == DIALOG_SAVE || type == DIALOG_SAVE_CLOSE)
 		{
@@ -166,6 +167,7 @@ static void collection_save_or_load_dialog(const gchar *path,
 		text = _("Save collection as:");
 		btntext = _("Save");
 		btnfunc = collection_save_cb;
+		stock_id = GTK_STOCK_SAVE;
 		}
 	else if (type == DIALOG_LOAD)
 		{
@@ -173,6 +175,7 @@ static void collection_save_or_load_dialog(const gchar *path,
 		text = _("Open collection from:");
 		btntext = _("Open");
 		btnfunc = collection_load_cb;
+		stock_id = GTK_STOCK_OPEN;
 		}
 	else
 		{
@@ -181,13 +184,14 @@ static void collection_save_or_load_dialog(const gchar *path,
 		text = _("Append collection from:");
 		btntext = _("Append");
 		btnfunc = collection_append_cb;
+		stock_id = GTK_STOCK_ADD;
 		}
 
 	if (cd) collection_ref(cd);
 
 	fd = file_util_file_dlg(title, text, "GQview", "dlg_collection",
 			     collection_save_or_load_dialog_close_cb, cd);
-	file_dialog_add_button(fd, btntext, btnfunc, TRUE);
+	file_dialog_add_button_stock(fd, btntext, stock_id, btnfunc, TRUE);
 
 	base = g_strconcat(homedir(), "/", GQVIEW_RC_DIR_COLLECTIONS, NULL);
 	file_dialog_add_path_widgets(fd, base, path,
