@@ -1,6 +1,6 @@
 /*
  * GQview
- * (C) 2002 John Ellis
+ * (C) 2004 John Ellis
  *
  * Author: John Ellis
  *
@@ -16,14 +16,16 @@
 #include "collect-dlg.h"
 #include "collect-io.h"
 #include "collect-table.h"
+#include "editors.h"
 #include "filelist.h"
 #include "img-view.h"
+#include "info.h"
 #include "layout.h"
 #include "layout_image.h"
 #include "thumb.h"
 #include "utilops.h"
-#include "ui_clist_edit.h"
 #include "ui_fileops.h"
+#include "ui_tree_edit.h"
 
 #include <gdk/gdkkeysyms.h> /* for keyboard values */
 
@@ -51,7 +53,7 @@ static void collection_window_close(CollectWindow *cw);
  *-------------------------------------------------------------------
  */
 
-CollectInfo *collection_info_new(const gchar *path, struct stat *st, GdkPixmap *pixmap, GdkPixmap *mask)
+CollectInfo *collection_info_new(const gchar *path, struct stat *st, GdkPixbuf *pixbuf)
 {
 	CollectInfo *ci;
 
@@ -62,27 +64,16 @@ CollectInfo *collection_info_new(const gchar *path, struct stat *st, GdkPixmap *
 	ci->size = st->st_size;
 	ci->date = st->st_mtime;
 
-	ci->pixmap = pixmap;
-	if (ci->pixmap) gdk_pixmap_ref(ci->pixmap);
-
-	ci->mask = mask;
-	if (ci->mask) gdk_bitmap_ref(ci->mask);
+	ci->pixbuf = pixbuf;
+	if (ci->pixbuf) g_object_ref(ci->pixbuf);
 
 	return ci;
 }
 
 void collection_info_free_thumb(CollectInfo *ci)
 {
-	if (ci->pixmap)
-		{
-		gdk_pixmap_unref(ci->pixmap);
-		ci->pixmap = NULL;
-		}
-	if (ci->mask)
-		{
-		gdk_bitmap_unref(ci->mask);
-		ci->mask = NULL;
-		}
+	if (ci->pixbuf) g_object_unref(ci->pixbuf);
+	ci->pixbuf = NULL;
 }
 
 void collection_info_free(CollectInfo *ci)
@@ -94,15 +85,11 @@ void collection_info_free(CollectInfo *ci)
 	g_free(ci);
 }
 
-void collection_info_set_thumb(CollectInfo *ci, GdkPixmap *pixmap, GdkPixmap *mask)
+void collection_info_set_thumb(CollectInfo *ci, GdkPixbuf *pixbuf)
 {
+	if (pixbuf) g_object_ref(pixbuf);
 	collection_info_free_thumb(ci);
-
-	ci->pixmap = pixmap;
-	if (ci->pixmap) gdk_pixmap_ref(ci->pixmap);
-
-	ci->mask = mask;
-	if (ci->mask) gdk_bitmap_ref(ci->mask);
+	ci->pixbuf = pixbuf;
 }
 
 gint collection_info_load_thumb(CollectInfo *ci)
@@ -159,7 +146,7 @@ static gint collection_list_sort_cb(gconstpointer a, gconstpointer b)
 			return 0;
 			break;
 		case SORT_PATH:
-			return strcmp(cia->path, cib->path);
+			return CASE_SORT(cia->path, cib->path);
 			break;
 #ifdef HAVE_STRVERSCMP
 		case SORT_NUMBER:
@@ -168,7 +155,7 @@ static gint collection_list_sort_cb(gconstpointer a, gconstpointer b)
 #endif
 		case SORT_NAME:
 		default:
-			return strcmp(filename_from_path(cia->path), filename_from_path(cib->path));
+			return CASE_SORT(filename_from_path(cia->path), filename_from_path(cib->path));
 			break;
 		}
 
@@ -572,10 +559,10 @@ gint collection_add(CollectionData *cd, const gchar *path, gint sorted)
 {
 	struct stat st;
 
-	if (stat(path, &st) >= 0 && !S_ISDIR(st.st_mode))
+	if (stat_utf8(path, &st) && !S_ISDIR(st.st_mode))
 		{
 		CollectInfo *ci;
-		ci = collection_info_new(path, &st, NULL, NULL);
+		ci = collection_info_new(path, &st, NULL);
 		cd->list = collection_list_add(cd->list, ci, sorted ? cd->sort_method : SORT_NONE);
 		cd->changed = TRUE;
 
@@ -600,10 +587,10 @@ gint collection_insert(CollectionData *cd, const gchar *path, CollectInfo *inser
 
 	if (!insert_ci) return collection_add(cd, path, sorted);
 
-	if (stat(path, &st) >= 0 && !S_ISDIR(st.st_mode))
+	if (stat_utf8(path, &st) >= 0 && !S_ISDIR(st.st_mode))
 		{
 		CollectInfo *ci;
-		ci = collection_info_new(path, &st, NULL, NULL);
+		ci = collection_info_new(path, &st, NULL);
 		cd->list = collection_list_insert(cd->list, ci, insert_ci, sorted ? cd->sort_method : SORT_NONE);
 		cd->changed = TRUE;
 
@@ -764,6 +751,12 @@ static gint collection_window_keypress(GtkWidget *widget, GdkEventKey *event, gp
 			case '8':
 				edit_val = 7;
 				break;
+			case '9':
+				edit_val = 8;
+				break;
+			case '0':
+				edit_val = 9;
+				break;
 			case 'A': case 'a':
 				if (event->state & GDK_SHIFT_MASK)
 					{
@@ -800,6 +793,10 @@ static gint collection_window_keypress(GtkWidget *widget, GdkEventKey *event, gp
 				file_util_delete(NULL, collection_table_selection_get_list(cw->table));
 				stop_signal = TRUE;
 				break;
+			case 'P': case 'p':
+				info_window_new(NULL, collection_table_selection_get_list(cw->table));
+				stop_signal = TRUE;
+				break;
 			case 'W': case 'w':
 				collection_window_close(cw);
 				stop_signal = TRUE;
@@ -944,7 +941,7 @@ static void collection_window_add(CollectWindow *cw, CollectInfo *ci)
 {
 	if (!cw) return;
 
-	if (!ci->pixmap) collection_load_thumb_idle(cw->cd);
+	if (!ci->pixbuf) collection_load_thumb_idle(cw->cd);
 	collection_table_file_add(cw->table, ci);
 }
 
@@ -952,7 +949,7 @@ static void collection_window_insert(CollectWindow *cw, CollectInfo *ci)
 {
 	if (!cw) return;
 
-	if (!ci->pixmap) collection_load_thumb_idle(cw->cd);
+	if (!ci->pixbuf) collection_load_thumb_idle(cw->cd);
 	collection_table_file_insert(cw->table, ci);
 	if (!cw) return;
 }
@@ -1050,8 +1047,8 @@ static void collection_close_dlg_show(CollectWindow *cw)
 				"GQview", "close_collection", FALSE,
 				collection_close_cancel_cb, cw);
 
-	generic_dialog_add(gd, _("Save"), collection_close_save_cb, TRUE);
-	generic_dialog_add(gd, _("Close"), collection_close_close_cb, FALSE);
+	generic_dialog_add_stock(gd, _("Save"), GTK_STOCK_SAVE, collection_close_save_cb, TRUE);
+	generic_dialog_add_stock(gd, _("Close"), GTK_STOCK_CLOSE, collection_close_close_cb, FALSE);
 
 	cw->close_dialog = gd->dialog;
 
@@ -1107,6 +1104,7 @@ CollectWindow *collection_window_new(const gchar *path)
 	GtkWidget *frame;
 	GtkWidget *status_label;
 	GtkWidget *extra_label;
+	GdkGeometry geometry;
 
 	cw = g_new0(CollectWindow, 1);
 
@@ -1119,6 +1117,14 @@ CollectWindow *collection_window_new(const gchar *path)
 	cw->window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
 	window_set_icon(cw->window, (const gchar **)collect_xpm, NULL);
 
+	geometry.min_width = 32;
+	geometry.min_height = 32;
+	geometry.base_width = COLLECT_DEF_WIDTH;
+	geometry.base_height = COLLECT_DEF_HEIGHT;
+	gtk_window_set_geometry_hints(GTK_WINDOW(cw->window), NULL, &geometry,
+				      GDK_HINT_MIN_SIZE | GDK_HINT_BASE_SIZE);
+
+
 	if (save_window_positions && path && collection_load_only_geometry(cw->cd, path))
 		{
 		/* FIXME: x, y is not implemented */
@@ -1129,16 +1135,16 @@ CollectWindow *collection_window_new(const gchar *path)
 		gtk_window_set_default_size(GTK_WINDOW(cw->window), COLLECT_DEF_WIDTH, COLLECT_DEF_HEIGHT);
 		}
 
-	gtk_window_set_policy(GTK_WINDOW(cw->window), TRUE, TRUE, FALSE);
+	gtk_window_set_resizable(GTK_WINDOW(cw->window), TRUE);
 	collection_window_update_title(cw);
 	gtk_window_set_wmclass(GTK_WINDOW(cw->window), "collection", "GQview");
-	gtk_container_border_width (GTK_CONTAINER (cw->window), 0);
+	gtk_container_set_border_width (GTK_CONTAINER (cw->window), 0);
 
-	gtk_signal_connect(GTK_OBJECT(cw->window), "delete_event",
-			   GTK_SIGNAL_FUNC(collection_window_delete), cw);
+	g_signal_connect(G_OBJECT(cw->window), "delete_event",
+			 G_CALLBACK(collection_window_delete), cw);
 
-	gtk_signal_connect(GTK_OBJECT(cw->window),"key_press_event",
-			   GTK_SIGNAL_FUNC(collection_window_keypress), cw);
+	g_signal_connect(G_OBJECT(cw->window),"key_press_event",
+			 G_CALLBACK(collection_window_keypress), cw);
 
 	vbox = gtk_vbox_new(FALSE, 0);
 	gtk_container_add(GTK_CONTAINER(cw->window), vbox);
@@ -1162,15 +1168,14 @@ CollectWindow *collection_window_new(const gchar *path)
 	gtk_widget_show(status_label);
 
 	extra_label = gtk_progress_bar_new();
-	gtk_progress_set_show_text(GTK_PROGRESS(extra_label), TRUE);
-	gtk_progress_configure(GTK_PROGRESS(extra_label), 0.0, 0.0, 1.0);
+	gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(extra_label), 0.0);
 	gtk_box_pack_start(GTK_BOX(cw->status_box), extra_label, TRUE, TRUE, 0);
 	gtk_widget_show(extra_label);
 
 	collection_table_set_labels(cw->table, status_label, extra_label);
 
 	gtk_widget_show(cw->window);
-	gtk_widget_grab_focus(cw->table->clist);
+	gtk_widget_grab_focus(cw->table->listview);
 
 	collection_set_update_info_func(cw->cd, collection_window_update_info, cw);
 
