@@ -1,6 +1,6 @@
 /*
  * GQview
- * (C) 2002 John Ellis
+ * (C) 2004 John Ellis
  *
  * Author: John Ellis
  *
@@ -14,12 +14,12 @@
 #define COLLECT_H
 
 
-CollectInfo *collection_info_new(const gchar *path, struct stat *st, GdkPixmap *pixmap, GdkPixmap *mask);
+CollectInfo *collection_info_new(const gchar *path, struct stat *st, GdkPixbuf *pixbuf);
 
 void collection_info_free_thumb(CollectInfo *ci);
 void collection_info_free(CollectInfo *ci);
 
-void collection_info_set_thumb(CollectInfo *ci, GdkPixmap *pixmap, GdkPixmap *mask);
+void collection_info_set_thumb(CollectInfo *ci, GdkPixbuf *pixbuf);
 gint collection_info_load_thumb(CollectInfo *ci);
 
 void collection_list_free(GList *list);
