@@ -1,6 +1,6 @@
 /*
  * GQview
- * (C) 2002 John Ellis
+ * (C) 2004 John Ellis
  *
  * Author: John Ellis
  *
@@ -14,6 +14,7 @@
 #include "filelist.h"
 
 #include "cache.h"
+#include "rcfile.h"
 #include "ui_fileops.h"
 
 
@@ -24,6 +25,7 @@
  */
 
 static GList *filter_list = NULL;
+static GList *extension_list = NULL;
 
 gint ishidden(const gchar *name)
 {
@@ -32,12 +34,238 @@ gint ishidden(const gchar *name)
 	return TRUE;
 }
 
+static FilterEntry *filter_entry_new(const gchar *key, const gchar *description,
+				     const gchar *extensions, gint enabled)
+{
+	FilterEntry *fe;
+
+	fe = g_new0(FilterEntry, 1);
+	fe->key = g_strdup(key);
+	fe->description = g_strdup(description);
+	fe->extensions = g_strdup(extensions);
+	fe->enabled = enabled;
+	
+	return fe;
+}
+
+static void filter_entry_free(FilterEntry *fe)
+{
+	if (!fe) return;
+
+	g_free(fe->key);
+	g_free(fe->description);
+	g_free(fe->extensions);
+	g_free(fe);
+}
+
+GList *filter_get_list(void)
+{
+	return filter_list;
+}
+
+void filter_remove_entry(FilterEntry *fe)
+{
+	if (!g_list_find(filter_list, fe)) return;
+
+	filter_list = g_list_remove(filter_list, fe);
+	filter_entry_free(fe);
+}
+
+static gint filter_key_exists(const gchar *key)
+{
+	GList *work;
+
+	if (!key) return FALSE;
+
+	work = filter_list;
+	while (work)
+		{
+		FilterEntry *fe = work->data;
+		work = work->next;
+
+		if (strcmp(fe->key, key) == 0) return TRUE;
+		}
+
+	return FALSE;
+}
+
+void filter_add(const gchar *key, const gchar *description, const gchar *extensions, gint enabled)
+{
+	filter_list = g_list_append(filter_list, filter_entry_new(key, description, extensions, enabled));
+}
+
+void filter_add_unique(const gchar *description, const gchar *extensions, gint enabled)
+{
+	gchar *key;
+	gint n;
+
+	key = g_strdup("user0");
+	n = 1;
+	while (filter_key_exists(key))
+		{
+		g_free(key);
+		if (n > 999) return;
+		key = g_strdup_printf("user%d", n);
+		n++;
+		}
+
+	filter_add(key, description, extensions, enabled);
+	g_free(key);
+}
+
+static void filter_add_if_missing(const gchar *key, const gchar *description, const gchar *extensions, gint enabled)
+{
+	GList *work;
+
+	if (!key) return;
+
+	work = filter_list;
+	while (work)
+		{
+		FilterEntry *fe = work->data;
+		work = work->next;
+		if (fe->key && strcmp(fe->key, key) == 0) return;
+		}
+
+	filter_add(key, description, extensions, enabled);
+}
+
+void filter_reset(void)
+{
+	GList *work;
+
+	work = filter_list;
+	while (work)
+		{
+		FilterEntry *fe = work->data;
+		work = work->next;
+		filter_entry_free(fe);
+		}
+
+	g_list_free(filter_list);
+	filter_list = NULL;
+}
+
+void filter_add_defaults(void)
+{
+	GSList *list, *work;
+
+	list = gdk_pixbuf_get_formats();
+	work = list;
+	while (work)
+		{
+		GdkPixbufFormat *format;
+		gchar *name;
+		gchar *desc;
+		gchar **extensions;
+		GString *filter = NULL;
+		gint i;
+		
+		format = work->data;
+		work = work->next;
+
+		name = gdk_pixbuf_format_get_name(format);
+		desc = gdk_pixbuf_format_get_description(format);
+		extensions = gdk_pixbuf_format_get_extensions(format);
+
+		i = 0;
+		while (extensions[i])
+			{
+			if (!filter)
+				{
+				filter = g_string_new(".");
+				filter = g_string_append(filter, extensions[i]);
+				}
+			else
+				{
+				filter = g_string_append(filter, ";.");
+				filter = g_string_append(filter, extensions[i]);
+				}
+			i++;
+			}
+
+		if (debug) printf("loader reported [%s] [%s] [%s]\n", name, desc, filter->str);
+
+		filter_add_if_missing(name, desc, filter->str, TRUE);
+
+		g_free(name);
+		g_free(desc);
+		g_strfreev(extensions);
+		g_string_free(filter, TRUE);
+		}
+	g_slist_free(list);
+
+	/* add defaults even if gdk-pixbuf does not have them, but disabled */
+	filter_add_if_missing("jpeg", "JPEG group", ".jpg;.jpeg;.jpe", FALSE);
+	filter_add_if_missing("png", "Portable Network Graphic", ".png", FALSE);
+	filter_add_if_missing("tiff", "Tiff", ".tif;.tiff", FALSE);
+	filter_add_if_missing("pnm", "Packed Pixel formats", ".pbm;.pgm;.pnm;.ppm", FALSE);
+	filter_add_if_missing("gif", "Graphics Interchange Format", ".gif", FALSE);
+	filter_add_if_missing("xbm", "X bitmap", ".xbm", FALSE);
+	filter_add_if_missing("xpm", "X pixmap", ".xpm", FALSE);
+	filter_add_if_missing("bmp", "Bitmap", ".bmp", FALSE);
+	filter_add_if_missing("ico", "Icon file", ".ico;.cur", FALSE);
+	filter_add_if_missing("ras", "Raster", ".ras", FALSE);
+	filter_add_if_missing("svg", "Scalable Vector Graphics", ".svg", FALSE);
+}
+
+static GList *filter_to_list(const gchar *extensions)
+{
+	GList *list = NULL;
+	const gchar *p;
+
+	if (!extensions) return NULL;
+
+	p = extensions;
+	while (*p != '\0')
+		{
+		const gchar *b;
+		gint l = 0;
+
+		b = p;
+		while (*p != '\0' && *p != ';')
+			{
+			p++;
+			l++;
+			}
+		list = g_list_append(list, g_strndup(b, l));
+		if (*p == ';') p++;
+		}
+
+	return list;
+}
+
+void filter_rebuild(void)
+{
+	GList *work;
+
+	path_list_free(extension_list);
+	extension_list = NULL;
+
+	work = filter_list;
+	while (work)
+		{
+		FilterEntry *fe;
+
+		fe = work->data;
+		work = work->next;
+
+		if (fe->enabled)
+			{
+			GList *ext;
+
+			ext = filter_to_list(fe->extensions);
+			if (ext) extension_list = g_list_concat(extension_list, ext);
+			}
+		}
+}
+
 gint filter_name_exists(const gchar *name)
 {
 	GList *work;
-	if (!filter_list || file_filter_disable) return TRUE;
+	if (!extension_list || file_filter_disable) return TRUE;
 
-	work = filter_list;
+	work = extension_list;
 	while (work)
 		{
 		gchar *filter = work->data;
@@ -53,49 +281,82 @@ gint filter_name_exists(const gchar *name)
 	return FALSE;
 }
 
-static void filter_add(gchar *text, gint add)
+void filter_write_list(FILE *f)
 {
-	if (add) filter_list = g_list_append(filter_list, g_strdup(text));
+	GList *work;
+
+	work = filter_list;
+	while (work)
+		{
+		FilterEntry *fe = work->data;
+		work = work->next;
+
+		fprintf(f, "filter_ext: \"%s%s\" \"%s\" \"%s\"\n", (fe->enabled) ? "" : "#",
+			fe->key, fe->extensions,
+			(fe->description) ? fe->description : "");
+		}
 }
 
-void filter_rebuild(void)
+void filter_parse(const gchar *text)
 {
-	if (filter_list)
+	const gchar *p;
+	gchar *key;
+	gchar *ext;
+	gchar *desc;
+	gint enabled = TRUE;
+
+	if (!text || text[0] != '"') return;
+
+	key = quoted_value(text);
+	if (!key) return;
+
+	p = text;
+	p++;
+	while (*p != '"' && *p != '\0') p++;
+	if (*p != '"')
 		{
-		path_list_free(filter_list);
-		filter_list = NULL;
+		g_free(key);
+		return;
+		}
+	p++;
+	while (*p != '"' && *p != '\0') p++;
+	if (*p != '"')
+		{
+		g_free(key);
+		return;
 		}
 
-	filter_add(".jpg", filter_include_jpg);
-	filter_add(".jpeg", filter_include_jpg);
-	filter_add(".xpm", filter_include_xpm);
-	filter_add(".tif", filter_include_tif);
-	filter_add(".tiff", filter_include_tif);
-	filter_add(".gif", filter_include_gif);
-	filter_add(".png", filter_include_png);
-	filter_add(".ppm", filter_include_ppm);
-	filter_add(".pgm", filter_include_pgm);
-	filter_add(".pcx", filter_include_pcx);
-	filter_add(".bmp", filter_include_bmp);
+	ext = quoted_value(p);
 
-	if (custom_filter)
+	p++;
+	while (*p != '"' && *p != '\0') p++;
+	if (*p == '"') p++;
+	while (*p != '"' && *p != '\0') p++;
+
+	if (*p == '"')
 		{
-		gchar *buf = g_strdup(custom_filter);
-		gchar *pos_ptr_b;
-		gchar *pos_ptr_e = buf;
-		while(pos_ptr_e[0] != '\0')
-			{
-			pos_ptr_b = pos_ptr_e;
-			while (pos_ptr_e[0] != ';' && pos_ptr_e[0] != '\0') pos_ptr_e++;
-			if (pos_ptr_e[0] == ';')
-				{
-				pos_ptr_e[0] = '\0';
-				pos_ptr_e++;
-				}
-			filter_add(pos_ptr_b, TRUE);
-			}
-		g_free(buf);
+		desc = quoted_value(p);
 		}
+	else
+		{
+		desc = NULL;
+		}
+
+	if (key && key[0] == '#')
+		{
+		gchar *tmp;
+		tmp = g_strdup(key + 1);
+		g_free(key);
+		key = tmp;
+
+		enabled = FALSE;
+		}
+
+	if (key && strlen(key) > 0 && ext) filter_add(key, desc, ext, enabled);
+
+	g_free(key);
+	g_free(ext);
+	g_free(desc);
 }
 
 GList *path_list_filter(GList *list, gint is_dir_list)
@@ -138,7 +399,7 @@ GList *path_list_filter(GList *list, gint is_dir_list)
 
 static gint path_list_sort_cb(gconstpointer a, gconstpointer b)
 {
-	return strcmp((gchar *)a, (gchar *)b);
+	return CASE_SORT((gchar *)a, (gchar *)b);
 }
 
 GList *path_list_sort(GList *list)
@@ -196,7 +457,7 @@ GList *path_list_recursive(const gchar *path)
  *-----------------------------------------------------------------------------
  */
 
-gchar *text_from_size(gint size)
+gchar *text_from_size(gint64 size)
 {
 	gchar *a, *b;
 	gchar *s, *d;
@@ -205,8 +466,15 @@ gchar *text_from_size(gint size)
 	/* what I would like to use is printf("%'d", size)
 	 * BUT: not supported on every libc :(
 	 */
-
-	a = g_strdup_printf("%d", size);
+	if (size > G_MAXUINT)
+		{
+		/* the %lld conversion is not valid in all libcs, so use a simple work-around */
+		a = g_strdup_printf("%d%09d", (guint)(size / 1000000000), (guint)(size % 1000000000));
+		}
+	else
+		{
+		a = g_strdup_printf("%d", (guint)size);
+		}
 	l = strlen(a);
 	n = (l - 1)/ 3;
 	if (n < 1) return a;
@@ -236,18 +504,24 @@ gchar *text_from_size(gint size)
 	return b;
 }
 
-gchar *text_from_size_abrev(gint size)
+gchar *text_from_size_abrev(gint64 size)
 {
-	if (size < 1024)
+	if (size < (gint64)1024)
 		{
-		return g_strdup_printf(_("%d bytes"), size);
+		return g_strdup_printf(_("%d bytes"), (gint)size);
 		}
-	if (size < 1048576)
+	if (size < (gint64)1048576)
 		{
 		return g_strdup_printf(_("%.1f K"), (gfloat)size / 1024.0);
 		}
+	if (size < (gint64)1073741824)
+		{
+		return g_strdup_printf(_("%.1f MB"), (gfloat)size / 1048576.0);
+		}
 
-	return g_strdup_printf(_("%.1f MB"), (gfloat)size / 1048576.0);
+	/* to avoid overflowing the float, do division in two steps */
+	size /= 1048576.0;
+	return g_strdup_printf(_("%.1f GB"), (gfloat)size / 1024.0);
 }
 
 const gchar *text_from_time(time_t t)
@@ -269,15 +543,16 @@ const gchar *text_from_time(time_t t)
  *-----------------------------------------------------------------------------
  */
 
-FileData *file_data_new(gchar *path, struct stat *st)
+FileData *file_data_new(const gchar *path, struct stat *st)
 {
 	FileData *fd;
 
 	fd = g_new0(FileData, 1);
-	fd->path = path;
+	fd->path = path_to_utf8(path);
 	fd->name = filename_from_path(fd->path);
 	fd->size = st->st_size;
 	fd->date = st->st_mtime;
+	fd->pixbuf = NULL;
 
 	return fd;
 }
@@ -285,6 +560,7 @@ FileData *file_data_new(gchar *path, struct stat *st)
 void file_data_free(FileData *fd)
 {
 	g_free(fd->path);
+	if (fd->pixbuf) g_object_unref(fd->pixbuf);
 	g_free(fd);
 }
 
@@ -327,7 +603,7 @@ static gint sort_file_cb(void *a, void *b)
 #endif
 		case SORT_NAME:
 		default:
-			return strcmp(fa->name, fb->name);
+			return CASE_SORT(fa->name, fb->name);
 			break;
 		}
 }
@@ -351,28 +627,35 @@ gint filelist_read(const gchar *path, GList **files, GList **dirs)
 	DIR *dp;
 	struct dirent *dir;
 	struct stat ent_sbuf;
+	gchar *pathl;
 	GList *dlist;
 	GList *flist;
 
 	dlist = NULL;
 	flist = NULL;
 
-	if (!path || (dp = opendir(path)) == NULL)
+	pathl = path_from_utf8(path);
+	if (!pathl || (dp = opendir(pathl)) == NULL)
 		{
+		g_free(pathl);
 		if (files) *files = NULL;
 		if (dirs) *dirs = NULL;
 		return FALSE;
 		}
 
 	/* root dir fix */
-	if (path[0] == '/' && path[1] == '\0') path = "";
+	if (pathl[0] == '/' && pathl[1] == '\0')
+		{
+		g_free(pathl);
+		pathl = g_strdup("");
+		}
 
 	while ((dir = readdir(dp)) != NULL)
 		{
 		gchar *name = dir->d_name;
 		if (show_dot_files || !ishidden(name))
 			{
-			gchar *filepath = g_strconcat(path, "/", name, NULL);
+			gchar *filepath = g_strconcat(pathl, "/", name, NULL);
 			if (stat(filepath, &ent_sbuf) >= 0)
 				{
 				if (S_ISDIR(ent_sbuf.st_mode))
@@ -383,7 +666,6 @@ gint filelist_read(const gchar *path, GList **files, GList **dirs)
 					    strcmp(name, GQVIEW_CACHE_DIR) != 0)
 						{
 						dlist = g_list_prepend(dlist, file_data_new(filepath, &ent_sbuf));
-						filepath = NULL;
 						}
 					}
 				else
@@ -391,7 +673,6 @@ gint filelist_read(const gchar *path, GList **files, GList **dirs)
 					if ((files) && filter_name_exists(name))
 						{
 						flist = g_list_prepend(flist, file_data_new(filepath, &ent_sbuf));
-						filepath = NULL;
 						}
 					}
 				}
@@ -401,6 +682,8 @@ gint filelist_read(const gchar *path, GList **files, GList **dirs)
 
 	closedir(dp);
 
+	g_free(pathl);
+
 	if (dirs) *dirs = dlist;
 	if (files) *files = flist;
 
