@@ -1,6 +1,6 @@
 /*
  * GQview
- * (C) 2002 John Ellis
+ * (C) 2004 John Ellis
  *
  * Author: John Ellis
  *
@@ -13,6 +13,8 @@
 #include "gqview.h"
 #include "image-load.h"
 
+#include "ui_fileops.h"
+
 #include <fcntl.h>
 
 
@@ -67,9 +69,9 @@ static void image_loader_stop(ImageLoader *il)
 	if (il->loader)
 		{
 		/* some loaders do not have a pixbuf till close, order is important here */
-		gdk_pixbuf_loader_close(il->loader);
+		gdk_pixbuf_loader_close(il->loader, NULL);
 		image_loader_sync_pixbuf(il);
-		gtk_object_unref(GTK_OBJECT(il->loader));
+		g_object_unref(G_OBJECT(il->loader));
 		il->loader = NULL;
 		}
 
@@ -134,7 +136,7 @@ static gint image_loader_idle_cb(gpointer data)
 			return FALSE;
 			}
 
-		if (b < 0 || (b > 0 && !gdk_pixbuf_loader_write(il->loader, buf, b)))
+		if (b < 0 || (b > 0 && !gdk_pixbuf_loader_write(il->loader, buf, b, NULL)))
 			{
 			image_loader_error(il);
 			return FALSE;
@@ -168,7 +170,7 @@ static gint image_loader_begin(ImageLoader *il)
 		return FALSE;
 		}
 
-	if (gdk_pixbuf_loader_write(il->loader, buf, b))
+	if (gdk_pixbuf_loader_write(il->loader, buf, b, NULL))
 		{
 		il->bytes_read += b;
 
@@ -191,7 +193,7 @@ static gint image_loader_begin(ImageLoader *il)
 			while(il->loader && !gdk_pixbuf_loader_get_pixbuf(il->loader) && b > 0)
 				{
 				b = read(il->load_fd, &buf, sizeof(buf));
-				if (b < 0 || (b > 0 && !gdk_pixbuf_loader_write(il->loader, buf, b)))
+				if (b < 0 || (b > 0 && !gdk_pixbuf_loader_write(il->loader, buf, b, NULL)))
 					{
 					image_loader_stop(il);
 					return FALSE;
@@ -234,10 +236,13 @@ static gint image_loader_begin(ImageLoader *il)
 static gint image_loader_setup(ImageLoader *il)
 {
 	struct stat st;
+	gchar *pathl;
 
 	if (!il) return FALSE;
 
-	il->load_fd = open(il->path, O_RDONLY | O_NONBLOCK);
+	pathl = path_from_utf8(il->path);
+	il->load_fd = open(pathl, O_RDONLY | O_NONBLOCK);
+	g_free(pathl);
 	if (il->load_fd == -1) return FALSE;
 
 	if (fstat(il->load_fd, &st) == 0)
@@ -246,8 +251,8 @@ static gint image_loader_setup(ImageLoader *il)
 		}
 
 	il->loader = gdk_pixbuf_loader_new();
-	gtk_signal_connect(GTK_OBJECT(il->loader), "area_updated",
-			   GTK_SIGNAL_FUNC(image_loader_area_cb), il);
+	g_signal_connect(G_OBJECT(il->loader), "area_updated",
+		 	 G_CALLBACK(image_loader_area_cb), il);
 
 	return image_loader_begin(il);
 }
