@@ -1,6 +1,6 @@
 /*
  * GQview
- * (C) 2002 John Ellis
+ * (C) 2004 John Ellis
  *
  * Author: John Ellis
  *
@@ -16,6 +16,7 @@
 
 #include "image-load.h"
 #include "collect.h"
+#include "exif.h"
 #include "pixbuf_util.h"
 #include "ui_fileops.h"
 
@@ -30,9 +31,14 @@
 /* size of the image loader buffer (512 bytes x defined number) */
 #define IMAGE_LOAD_BUFFER_COUNT 8
 
-/* define this so that more bytes are read per idle loop on larger images (> 1MB)
- * #define IMAGE_THROTTLE_LARGER_IMAGES 1
- */
+/* define this so that more bytes are read per idle loop on larger images (> 1MB) */
+#define IMAGE_THROTTLE_LARGER_IMAGES 1
+
+/* throttle factor to increase read bytes by (2 is double, 3 is triple, etc.) */
+#define IMAGE_THROTTLE_FACTOR 4
+
+/* the file size at which throttling take place */
+#define IMAGE_THROTTLE_THRESHOLD 1048576
 
 /* distance to drag mouse to disable image flip */
 #define IMAGE_DRAG_SCROLL_THRESHHOLD 4
@@ -42,6 +48,13 @@
 #define IMAGE_ALPHA_CHECK2 0x00666666
 #define IMAGE_ALPHA_CHECK_SIZE 16
 
+#define IMAGE_AUTO_REFRESH_TIME 3000
+
+/* when scaling image to below this size, use nearest pixel for scaling
+ * (below about 4, the other scale types become slow generating their conversion tables)
+ */
+#define IMAGE_MIN_SCALE_SIZE 8
+
 
 typedef enum {
 	TILE_RENDER_NONE = 0,	/* do nothing */
@@ -97,8 +110,9 @@ static void image_queue_clear(ImageWindow *imd);
 
 static void image_update_title(ImageWindow *imd);
 static void image_update_util(ImageWindow *imd);
+static void image_complete_util(ImageWindow *imd, gint preload);
 
-static void image_button_do(ImageWindow *imd, GdkEventButton *event, gint button);
+static void image_button_do(ImageWindow *imd, GdkEvent *event, gint button);
 
 
 /*
@@ -111,7 +125,8 @@ static gint pixmap_calc_size(GdkPixmap *pixmap)
 {
 	gint w, h, d;
 
-	gdk_window_get_geometry(pixmap, NULL, NULL, &w, &h, &d);
+	d = gdk_drawable_get_depth(pixmap);
+	gdk_drawable_get_size(pixmap, &w, &h);
 	return w * h * (d / 8);
 }
 
@@ -356,6 +371,8 @@ static void image_tile_render(ImageWindow *imd, ImageTile *it,
 		return;
 		}
 
+	if (!fast) it->render_todo = TILE_RENDER_NONE;
+
 	if (new_data) it->blank = FALSE;
 
 	image_tile_prepare(imd, it);
@@ -403,6 +420,11 @@ static void image_tile_render(ImageWindow *imd, ImageTile *it,
 		scale_x = (double)imd->width / imd->image_width;
 		scale_y = (double)imd->height / imd->image_height;
 
+		/* HACK: The pixbuf scalers get kinda buggy(crash) with extremely
+		 * small sizes for anything but GDK_INTERP_NEAREST
+		 */
+		if (imd->width < IMAGE_MIN_SCALE_SIZE || imd->height < IMAGE_MIN_SCALE_SIZE) fast = TRUE;
+
 		if (GDK_PIXBUF_TILES_BROKEN)
 			{
 			simple = (!has_alpha && (GdkInterpType)zoom_quality == GDK_INTERP_NEAREST);
@@ -442,8 +464,6 @@ static void image_tile_render(ImageWindow *imd, ImageTile *it,
 					      w, h,
 					      (GdkRgbDither)dither_quality, it->x + x, it->y + y);
 		}
-
-	if (!fast) it->render_todo = TILE_RENDER_NONE;
 }
 
 static void image_tile_expose(ImageWindow *imd, ImageTile *it,
@@ -478,6 +498,8 @@ static gint image_queue_draw_idle_cb(gpointer data)
 
 	if (!imd->pixbuf || (!imd->draw_queue && !imd->draw_queue_2pass) || imd->draw_idle_id == -1)
 		{
+		if (!imd->completed) image_complete_util(imd, FALSE);
+
 		imd->draw_idle_id = -1;
 		return FALSE;
 		}
@@ -539,6 +561,8 @@ static gint image_queue_draw_idle_cb(gpointer data)
 
 	if (!imd->draw_queue && !imd->draw_queue_2pass)
 		{
+		if (!imd->completed) image_complete_util(imd, FALSE);
+
 		imd->draw_idle_id = -1;
 		return FALSE;
 		}
@@ -874,7 +898,11 @@ static gint image_zoom_clamp(ImageWindow *imd, gfloat zoom, gint force, gint new
 	w = imd->image_width;
 	h = imd->image_height;
 
-	if (zoom == 0.0)
+	if (zoom == 0.0 && !imd->pixbuf)
+		{
+		scale = 1.0;
+		}
+	else if (zoom == 0.0)
 		{
 		gint max_w;
 		gint max_h;
@@ -988,13 +1016,16 @@ static void image_size_sync(ImageWindow *imd, gint new_width, gint new_height)
 	image_size_clamp(imd);
 	image_scroll_clamp(imd);
 
-	gtk_widget_set_usize(imd->image, imd->window_width, imd->window_height);
+	gtk_widget_set_size_request(imd->image, imd->window_width, imd->window_height);
 
 	/* clear any borders */
 	image_border_clear(imd);
 	
 	image_tile_sync(imd, imd->width, imd->height, FALSE);
+#if 0
+	/* no longer needed? (expose event should be doing this for us) */
 	image_redraw(imd, FALSE);
+#endif
 
 	if (imd->title_show_zoom) image_update_title(imd);
 	image_update_util(imd);
@@ -1029,11 +1060,12 @@ static void image_update_title(ImageWindow *imd)
 		g_free(buf);
 		}
 
-	title = g_strdup_printf("%s%s%s%s%s",
+	title = g_strdup_printf("%s%s%s%s%s%s",
 		imd->title ? imd->title : "",
 		imd->image_name ? imd->image_name : "",
 		zoom ? zoom : "",
 		collection ? collection : "",
+		imd->image_name ? " - " : "",
 		imd->title_right ? imd->title_right : "");
 
 	gtk_window_set_title(GTK_WINDOW(imd->top_window), title);
@@ -1045,7 +1077,17 @@ static void image_update_title(ImageWindow *imd)
 
 static void image_update_util(ImageWindow *imd)
 {
-	if (imd->update_func) imd->update_func(imd, imd->update_data);
+	if (imd->func_update) imd->func_update(imd, imd->data_update);
+}
+
+static void image_complete_util(ImageWindow *imd, gint preload)
+{
+	if (debug) printf("image load completed \"%s\" (%s)\n",
+			  (preload) ? imd->read_ahead_path : imd->image_path,
+			  (preload) ? "preload" : "current");
+
+	if (!preload) imd->completed = TRUE;
+	if (imd->func_complete) imd->func_complete(imd, preload, imd->data_complete);
 }
 
 static void image_scroll_real(ImageWindow *imd, gint x, gint y)
@@ -1080,7 +1122,6 @@ static void image_scroll_real(ImageWindow *imd, gint x, gint y)
 		gint x1, y1;
 		gint x2, y2;
 		GdkGC *gc;
-		GdkEvent *event;
 
 		if (x_off < 0)
 			{
@@ -1121,17 +1162,6 @@ static void image_scroll_real(ImageWindow *imd, gint x, gint y)
 		 * then be in a different screen location).
 		 */
 
-		while ((event = gdk_event_get_graphics_expose(imd->image->window)) != NULL)
-			{
-			gtk_widget_event(imd->image, event);
-			if (event->expose.count == 0)
-				{
-				gdk_event_free(event);
-				break;
-				}
-			gdk_event_free(event);
-			}
-
 		w = imd->vis_width - w;
 		h = imd->vis_height - h;
 
@@ -1170,7 +1200,7 @@ static void widget_set_cursor(GtkWidget *widget, gint icon)
 
 	gdk_window_set_cursor(widget->window, cursor);
 
-	if (cursor) gdk_cursor_destroy(cursor);
+	if (cursor) gdk_cursor_unref(cursor);
 }
 
 /*
@@ -1249,6 +1279,7 @@ static void image_pixbuf_sync(ImageWindow *imd, gfloat zoom, gint blank, gint ne
 		/* no pixbuf so just clear the window */
 		imd->image_width = 0;
 		imd->image_height = 0;
+		imd->scale = 1.0;
 
 		if (imd->image->window) gdk_window_clear(imd->image->window);
 
@@ -1269,14 +1300,14 @@ static void image_pixbuf_sync(ImageWindow *imd, gfloat zoom, gint blank, gint ne
 
 static void image_set_pixbuf(ImageWindow *imd, GdkPixbuf *pixbuf, gfloat zoom, gint new)
 {
-	if (pixbuf) gdk_pixbuf_ref(pixbuf);
-	if (imd->pixbuf) gdk_pixbuf_unref(imd->pixbuf);
+	if (pixbuf) g_object_ref(pixbuf);
+	if (imd->pixbuf) g_object_unref(imd->pixbuf);
 	imd->pixbuf = pixbuf;
 
 	image_pixbuf_sync(imd, zoom, FALSE, new);
 }
 
-static void image_alter_real(ImageWindow *imd, AlterType type)
+static void image_alter_real(ImageWindow *imd, AlterType type, gint clamp)
 {
 	GdkPixbuf *new = NULL;
 	gint x, y;
@@ -1322,16 +1353,88 @@ static void image_alter_real(ImageWindow *imd, AlterType type)
 			break;
 		}
 
-	if (new)
+	if (!new) return;
+
+	if (clamp)
 		{
 		image_set_pixbuf(imd, new, imd->zoom, TRUE);
-		gdk_pixbuf_unref(new);
+		g_object_unref(new);
 
 		if (imd->zoom != 0.0)
 			{
 			image_scroll(imd, x - (imd->vis_width / 2), y - (imd->vis_height / 2));
 			}
 		}
+	else
+		{
+		g_object_unref(imd->pixbuf);
+		imd->pixbuf = new;
+		}
+}
+
+static void image_post_process(ImageWindow *imd, gint clamp)
+{
+	if (exif_rotate_enable && imd->pixbuf)
+		{
+		ExifData *ed;
+		gint orientation;
+
+		ed = exif_read(imd->image_path);
+		if (ed && exif_get_integer(ed, "Orientation", &orientation))
+			{
+			/* see http://jpegclub.org/exif_orientation.html 
+			  1        2       3      4         5            6           7          8
+
+			888888  888888      88  88      8888888888  88                  88  8888888888
+			88          88      88  88      88  88      88  88          88  88      88  88
+			8888      8888    8888  8888    88          8888888888  8888888888          88
+			88          88      88  88
+			88          88  888888  888888
+			*/
+
+			switch (orientation)
+				{
+				case EXIF_ORIENTATION_TOP_LEFT:
+					/* normal -- nothing to do */
+					break;
+				case EXIF_ORIENTATION_TOP_RIGHT:
+					/* mirrored */
+					imd->delay_alter_type = ALTER_MIRROR;
+					break;
+				case EXIF_ORIENTATION_BOTTOM_RIGHT:
+					/* upside down */
+					imd->delay_alter_type = ALTER_ROTATE_180;
+					break;
+				case EXIF_ORIENTATION_BOTTOM_LEFT:
+					/* flipped */
+					imd->delay_alter_type = ALTER_FLIP;
+					break;
+				case EXIF_ORIENTATION_LEFT_TOP:
+					/* not implemented -- too wacky to fix in one step */
+					break;
+				case EXIF_ORIENTATION_RIGHT_TOP:
+					/* rotated -90 (270) */
+					imd->delay_alter_type = ALTER_ROTATE_90;
+					break;
+				case EXIF_ORIENTATION_RIGHT_BOTTOM:
+					/* not implemented -- too wacky to fix in one step */
+					break;
+				case EXIF_ORIENTATION_LEFT_BOTTOM:
+					/* rotated 90 */
+					imd->delay_alter_type = ALTER_ROTATE_90_CC;
+					break;
+				default:
+					/* The other values are out of range */
+					break;
+				}
+			}
+		exif_free(ed);
+		}
+
+	if (imd->delay_alter_type != ALTER_NONE)
+		{
+		image_alter_real(imd, imd->delay_alter_type, clamp);
+		}
 }
 
 /*
@@ -1347,7 +1450,7 @@ static void image_read_ahead_cancel(ImageWindow *imd)
 	image_loader_free(imd->read_ahead_il);
 	imd->read_ahead_il = NULL;
 
-	if (imd->read_ahead_pixbuf) gdk_pixbuf_unref(imd->read_ahead_pixbuf);
+	if (imd->read_ahead_pixbuf) g_object_unref(imd->read_ahead_pixbuf);
 	imd->read_ahead_pixbuf = NULL;
 
 	g_free(imd->read_ahead_path);
@@ -1363,7 +1466,7 @@ static void image_read_ahead_done_cb(ImageLoader *il, gpointer data)
 	imd->read_ahead_pixbuf = image_loader_get_pixbuf(imd->read_ahead_il);
 	if (imd->read_ahead_pixbuf)
 		{
-		gdk_pixbuf_ref(imd->read_ahead_pixbuf);
+		g_object_ref(imd->read_ahead_pixbuf);
 		}
 	else
 		{
@@ -1371,6 +1474,8 @@ static void image_read_ahead_done_cb(ImageLoader *il, gpointer data)
 		}
 	image_loader_free(imd->read_ahead_il);
 	imd->read_ahead_il = NULL;
+
+	image_complete_util(imd, TRUE);
 }
 
 static void image_read_ahead_error_cb(ImageLoader *il, gpointer data)
@@ -1395,6 +1500,7 @@ static void image_read_ahead_start(ImageWindow *imd)
 	if (!image_loader_start(imd->read_ahead_il, image_read_ahead_done_cb, imd))
 		{
 		image_read_ahead_cancel(imd);
+		image_complete_util(imd, TRUE);
 		}
 }
 
@@ -1411,6 +1517,57 @@ static void image_read_ahead_set(ImageWindow *imd, const gchar *path)
 	image_read_ahead_start(imd);
 }
 
+/*
+ *-------------------------------------------------------------------
+ * post buffering
+ *-------------------------------------------------------------------
+ */
+
+static void image_post_buffer_set(ImageWindow *imd, const gchar *path, GdkPixbuf *pixbuf)
+{
+	g_free(imd->prev_path);
+	if (imd->prev_pixbuf) g_object_unref(imd->prev_pixbuf);
+
+	if (path && pixbuf)
+		{
+		imd->prev_path = g_strdup(path);
+			
+		g_object_ref(pixbuf);
+		imd->prev_pixbuf = pixbuf;
+		}
+	else
+		{
+		imd->prev_path = NULL;
+		imd->prev_pixbuf = NULL;
+		}
+
+	if (debug) printf("post buffer set: %s\n", path);
+}
+
+static gint image_post_buffer_get(ImageWindow *imd)
+{
+	gint success;
+
+	if (!imd->pixbuf && imd->prev_pixbuf &&
+	    imd->image_path && imd->prev_path && strcmp(imd->image_path, imd->prev_path) == 0)
+		{
+		imd->pixbuf = imd->prev_pixbuf;
+		success = TRUE;
+		}
+	else
+		{
+		if (imd->prev_pixbuf) g_object_unref(imd->prev_pixbuf);
+		success = FALSE;
+		}
+
+	imd->prev_pixbuf = NULL;
+
+	g_free(imd->prev_path);
+	imd->prev_path = NULL;
+
+	return success;
+}
+
 /*
  *-------------------------------------------------------------------
  * loading
@@ -1423,7 +1580,7 @@ static void image_load_pixbuf_ready(ImageWindow *imd)
 
 	imd->pixbuf = image_loader_get_pixbuf(imd->il);
 
-	if (imd->pixbuf) gdk_pixbuf_ref(imd->pixbuf);
+	if (imd->pixbuf) g_object_ref(imd->pixbuf);
 
 	image_pixbuf_sync(imd, imd->zoom, TRUE, TRUE);
 }
@@ -1468,10 +1625,7 @@ static void image_load_done_cb(ImageLoader *il, gpointer data)
 	image_loader_free(imd->il);
 	imd->il = NULL;
 
-	if (imd->delay_alter_type != ALTER_NONE)
-		{
-		image_alter_real(imd, imd->delay_alter_type);
-		}
+	image_post_process(imd, TRUE);
 
 	image_read_ahead_start(imd);
 }
@@ -1489,14 +1643,14 @@ static void image_load_error_cb(ImageLoader *il, gpointer data)
 #ifdef IMAGE_THROTTLE_LARGER_IMAGES
 static void image_load_buffer_throttle(ImageLoader *il)
 {
-	if (!il || il->bytes_total < 1024 * 1024) return;
+	if (!il || il->bytes_total < IMAGE_THROTTLE_THRESHOLD) return;
 
 	/* Larger image files usually have larger chunks of data per pixel...
 	 * So increase the buffer read size so that the rendering chunks called
 	 * are also larger.
 	 */
 
-	image_loader_set_buffer_size(il, IMAGE_LOAD_BUFFER_COUNT * 8);
+	image_loader_set_buffer_size(il, IMAGE_LOAD_BUFFER_COUNT * IMAGE_THROTTLE_FACTOR);
 }
 #endif
 
@@ -1531,7 +1685,7 @@ static gint image_read_ahead_check(ImageWindow *imd)
 		imd->il->func_done = image_load_done_cb;
 
 		imd->pixbuf = image_loader_get_pixbuf(imd->il);
-		if (imd->pixbuf) gdk_pixbuf_ref(imd->pixbuf);
+		if (imd->pixbuf) g_object_ref(imd->pixbuf);
 
 		image_read_ahead_cancel(imd);
 		return TRUE;
@@ -1542,6 +1696,8 @@ static gint image_read_ahead_check(ImageWindow *imd)
 		imd->read_ahead_pixbuf = NULL;
 
 		image_read_ahead_cancel(imd);
+
+		image_post_process(imd, FALSE);
 		return TRUE;
 		}
 
@@ -1555,6 +1711,16 @@ static gint image_load_begin(ImageWindow *imd, const gchar *path)
 
 	if (imd->il) return FALSE;
 
+	imd->completed = FALSE;
+
+	if (image_post_buffer_get(imd))
+		{
+		if (debug) printf("from post buffer: %s\n", imd->image_path);
+
+		image_pixbuf_sync(imd, imd->zoom, FALSE, TRUE);
+		return TRUE;
+		}
+
 	if (image_read_ahead_check(imd))
 		{
 		if (debug) printf("from read ahead buffer: %s\n", imd->image_path);
@@ -1575,6 +1741,9 @@ static gint image_load_begin(ImageWindow *imd, const gchar *path)
 
 		image_loader_free(imd->il);
 		imd->il = NULL;
+
+		image_complete_util(imd, FALSE);
+
 		return FALSE;
 		}
 
@@ -1610,14 +1779,15 @@ static void image_change_complete(ImageWindow *imd, gfloat zoom, gint new)
 {
 	gint sync = TRUE;
 
-	if (imd->pixbuf) gdk_pixbuf_unref(imd->pixbuf);
+	if (imd->pixbuf) g_object_unref(imd->pixbuf);
 	imd->pixbuf = NULL;
 
 	imd->zoom = zoom;	/* store the zoom, needed by the loader */
 
+	image_reset(imd);
+
 	if (imd->image_path && isfile(imd->image_path))
 		{
-		image_reset(imd);
 		image_load_begin(imd, imd->image_path);
 		if (imd->pixbuf)
 			{
@@ -1630,16 +1800,19 @@ static void image_change_complete(ImageWindow *imd, gfloat zoom, gint new)
 			imd->unknown = TRUE;
 			}
 		imd->size = filesize(imd->image_path);
+		imd->mtime = filetime(imd->image_path);
 		}
 	else
 		{
 		if (imd->image_path)
 			{
 			imd->pixbuf = gdk_pixbuf_new_from_xpm_data(img_unknown);
+			imd->mtime = filetime(imd->image_path);
 			}
 		else
 			{
 			imd->pixbuf = NULL;
+			imd->mtime = 0;
 			}
 		imd->unknown = TRUE;
 		imd->size = 0;
@@ -1658,13 +1831,44 @@ static void image_change_complete(ImageWindow *imd, gfloat zoom, gint new)
 static void image_change_real(ImageWindow *imd, const gchar *path,
 			      CollectionData *cd, CollectInfo *info, gfloat zoom)
 {
+	GdkPixbuf *prev_pixbuf = NULL;
+	gchar *prev_path = NULL;
+	gint prev_clear = FALSE;
+
 	imd->collection = cd;
 	imd->collection_info = info;
-
+#if 0
 	if (!path) zoom = 1.0;
+#endif
+
+	if (enable_read_ahead && imd->image_path && imd->pixbuf)
+		{
+		if (imd->il)
+			{
+			/* current image is not finished */
+			prev_clear = TRUE;
+			}
+		else
+			{
+			prev_path = g_strdup(imd->image_path);
+			prev_pixbuf = imd->pixbuf;
+			g_object_ref(prev_pixbuf);
+			}
+		}
 
 	image_set_path(imd, path);
 	image_change_complete(imd, zoom, TRUE);
+
+	if (prev_pixbuf)
+		{
+		image_post_buffer_set(imd, prev_path, prev_pixbuf);
+		g_free(prev_path);
+		g_object_unref(prev_pixbuf);
+		}
+	else if (prev_clear)
+		{
+		image_post_buffer_set(imd, NULL, NULL);
+		}
 }
 
 /*
@@ -1710,43 +1914,47 @@ static gint image_size_cb(GtkWidget *widget, GtkAllocation *allocation, gpointer
  *-------------------------------------------------------------------
  */
 
-static void image_focus_paint(ImageWindow *imd)
+static void image_focus_paint(ImageWindow *imd, gint has_focus, GdkRectangle *area)
 {
 	GtkWidget *widget;
 
 	widget = imd->widget;
 	if (!widget->window) return;
 
-	gtk_paint_focus (widget->style, widget->window,
-			 NULL, widget, "image_window",
-			 widget->allocation.x, widget->allocation.y,
-			 widget->allocation.width - 1, widget->allocation.height - 1);	
+	if (has_focus)
+		{
+		gtk_paint_focus (widget->style, widget->window, GTK_STATE_ACTIVE,
+				 area, widget, "image_window",
+				 widget->allocation.x, widget->allocation.y,
+				 widget->allocation.width - 1, widget->allocation.height - 1);	
+		}
+	else
+		{
+		gtk_paint_shadow (widget->style, widget->window, GTK_STATE_NORMAL, GTK_SHADOW_IN,
+				  area, widget, "image_window",
+				  widget->allocation.x, widget->allocation.y,
+				  widget->allocation.width - 1, widget->allocation.height - 1);
+		}
 }
 
 static gint image_focus_expose(GtkWidget *widget, GdkEventExpose *event, gpointer data)
 {
 	ImageWindow *imd = data;
 
-	if (!GTK_WIDGET_HAS_FOCUS(widget)) return FALSE;
-
-	image_focus_paint(imd);
+	image_focus_paint(imd, GTK_WIDGET_HAS_FOCUS(widget), &event->area);
 	return TRUE;
 }
 
-static void image_focus_draw(GtkWidget *widget, GdkRectangle *area, gpointer data)
-{
-	ImageWindow *imd = data;
-
-	if (!GTK_WIDGET_HAS_FOCUS(widget)) return;
-	image_focus_paint(imd);
-}
-
 static gint image_focus_in_cb(GtkWidget *widget, GdkEventFocus *event, gpointer data)
 {
 	ImageWindow *imd = data;
 
 	GTK_WIDGET_SET_FLAGS(imd->widget, GTK_HAS_FOCUS);
+	image_focus_paint(imd, TRUE, NULL);
+
+#if 0
 	gtk_frame_set_shadow_type(GTK_FRAME(imd->widget), GTK_SHADOW_NONE);
+#endif
 
 	return TRUE;
 }
@@ -1756,7 +1964,11 @@ static gint image_focus_out_cb(GtkWidget *widget, GdkEventFocus *event, gpointer
 	ImageWindow *imd = data;
 
 	GTK_WIDGET_UNSET_FLAGS(imd->widget, GTK_HAS_FOCUS);
+	image_focus_paint(imd, FALSE, NULL);
+
+#if 0
 	gtk_frame_set_shadow_type(GTK_FRAME(imd->widget), GTK_SHADOW_IN);
+#endif
 
 	return TRUE;
 }
@@ -1811,9 +2023,7 @@ static gint image_mouse_press_cb(GtkWidget *widget, GdkEventButton *bevent, gpoi
 			imd->drag_moved = 0;
 			break;
 		case 3:
-		case 4:
-		case 5:
-			image_button_do(imd, bevent, bevent->button);
+			image_button_do(imd, (GdkEvent *)bevent, bevent->button);
 			break;
 		default:
 			break;
@@ -1837,7 +2047,7 @@ static gint image_mouse_release_cb(GtkWidget *widget, GdkEventButton *bevent, gp
 
 	if (bevent->button == 1 || bevent->button == 2)
 		{
-		if (imd->drag_moved < IMAGE_DRAG_SCROLL_THRESHHOLD) image_button_do(imd, bevent, bevent->button);
+		if (imd->drag_moved < IMAGE_DRAG_SCROLL_THRESHHOLD) image_button_do(imd, (GdkEvent *)bevent, bevent->button);
 		}
 
 	imd->in_drag = FALSE;
@@ -1845,6 +2055,24 @@ static gint image_mouse_release_cb(GtkWidget *widget, GdkEventButton *bevent, gp
 	return FALSE;
 }
 
+static gint image_scroll_cb(GtkWidget *widget, GdkEventScroll *event, gpointer data)
+{
+	ImageWindow *imd = data;
+
+	if (event->direction == GDK_SCROLL_UP)
+		{
+		image_button_do(imd, (GdkEvent *)event, 4);
+		return TRUE;
+		}
+	else if (event->direction == GDK_SCROLL_DOWN)
+		{
+		image_button_do(imd, (GdkEvent *)event, 5);
+		return TRUE;
+		}
+
+	return FALSE;
+}
+
 static void image_mouse_drag_cb(GtkWidget *widget, GdkDragContext *context, gpointer data)
 {
 	ImageWindow *imd = data;
@@ -1878,16 +2106,25 @@ void image_attach_window(ImageWindow *imd, GtkWidget *window,
 }
 
 void image_set_update_func(ImageWindow *imd,
-			   void (*update_func)(ImageWindow *imd, gpointer data),
+			   void (*func)(ImageWindow *imd, gpointer data),
 			   gpointer data)
 {
-	imd->update_func = update_func;
-	imd->update_data = data;
+	imd->func_update = func;
+	imd->data_update = data;
 }
 
-static void image_button_do(ImageWindow *imd, GdkEventButton *event, gint button)
+void image_set_complete_func(ImageWindow *imd,
+			     void (*func)(ImageWindow *, gint preload, gpointer),
+			     gpointer data)
 {
-	void (*func)(ImageWindow *, GdkEventButton *, gpointer);
+	imd->func_complete = func;
+	imd->data_complete = data;
+}
+
+
+static void image_button_do(ImageWindow *imd, GdkEvent *event, gint button)
+{
+	void (*func)(ImageWindow *, guint32 time, gdouble x, gdouble y, guint state, gpointer);
 	gpointer data;
 
 	switch (button)
@@ -1918,11 +2155,23 @@ static void image_button_do(ImageWindow *imd, GdkEventButton *event, gint button
 			break;
 		}
 
-	if (func) func(imd, event, data);
+	if (func)
+		{
+		if (event->type == GDK_BUTTON_PRESS || event->type == GDK_BUTTON_RELEASE)
+			{
+			GdkEventButton *bevent = (GdkEventButton *)event;
+			func(imd, bevent->time, bevent->x, bevent->y, bevent->state, data);
+			}
+		else if (event->type == GDK_SCROLL)
+			{
+			GdkEventScroll *sevent = (GdkEventScroll *)event;
+			func(imd, sevent->time, sevent->x, sevent->y, sevent->state, data);
+			}
+		}
 }
 
 void image_set_button_func(ImageWindow *imd, gint button,
-			   void (*func)(ImageWindow *, GdkEventButton *, gpointer),
+			   void (*func)(ImageWindow *, guint32 time, gdouble x, gdouble y, guint state, gpointer),
 			   gpointer data)
 {
 	switch (button)
@@ -2028,6 +2277,8 @@ static void image_loader_sync_data(ImageLoader *il, gpointer data)
  */
 void image_change_from_image(ImageWindow *imd, ImageWindow *source)
 {
+	if (imd == source) return;
+
 	imd->unknown = source->unknown;
 
 	image_set_pixbuf(imd, source->pixbuf, image_zoom_get(source), TRUE);
@@ -2035,6 +2286,7 @@ void image_change_from_image(ImageWindow *imd, ImageWindow *source)
 	imd->collection = source->collection;
 	imd->collection_info = source->collection_info;
 	imd->size = source->size;
+	imd->mtime = source->mtime;
 
 	image_set_path(imd, image_get_path(source));
 
@@ -2052,16 +2304,27 @@ void image_change_from_image(ImageWindow *imd, ImageWindow *source)
 		source->delay_alter_type = ALTER_NONE;
 		}
 
+	image_loader_free(imd->read_ahead_il);
 	imd->read_ahead_il = source->read_ahead_il;
 	source->read_ahead_il = NULL;
 	if (imd->read_ahead_il) image_loader_sync_data(imd->read_ahead_il, imd);
 
+	if (imd->read_ahead_pixbuf) g_object_unref(imd->read_ahead_pixbuf);
 	imd->read_ahead_pixbuf = source->read_ahead_pixbuf;
 	source->read_ahead_pixbuf = NULL;
 
+	g_free(imd->read_ahead_path);
 	imd->read_ahead_path = source->read_ahead_path;
 	source->read_ahead_path = NULL;
 
+	if (imd->prev_pixbuf) g_object_unref(imd->prev_pixbuf);
+	imd->prev_pixbuf = source->prev_pixbuf;
+	source->prev_pixbuf = NULL;
+
+	g_free(imd->prev_path);
+	imd->prev_path = source->prev_path;
+	source->prev_path = NULL;
+
 	imd->x_scroll = source->x_scroll;
 	imd->y_scroll = source->y_scroll;
 
@@ -2089,7 +2352,7 @@ void image_alter(ImageWindow *imd, AlterType type)
 		return;
 		}
 
-	image_alter_real(imd, type);
+	image_alter_real(imd, type, TRUE);
 }
 
 /* zoom */
@@ -2240,6 +2503,46 @@ void image_prebuffer_set(ImageWindow *imd, const gchar *path)
 		}
 }
 
+static gint image_auto_refresh_cb(gpointer data)
+{
+	ImageWindow *imd = data;
+	time_t newtime;
+	
+	if (!imd || !imd->pixbuf ||
+	    imd->il || !imd->image_path ||
+	    !update_on_time_change) return TRUE;
+
+	newtime = filetime(imd->image_path);
+	if (newtime > 0 && newtime != imd->mtime)
+		{
+		imd->mtime = newtime;
+		image_reload(imd);
+		}
+
+	return TRUE;
+}
+
+/* image auto refresh on time stamp change, in 1/1000's second, -1 disables */
+
+void image_auto_refresh(ImageWindow *imd, gint interval)
+{
+	if (!imd) return;
+
+	if (imd->auto_refresh_id > -1)
+		{
+		gtk_timeout_remove(imd->auto_refresh_id);
+		imd->auto_refresh_id = -1;
+		imd->auto_refresh_interval = -1;
+		}
+
+	if (interval < 0) return;
+
+	if (interval == 0) interval = IMAGE_AUTO_REFRESH_TIME;
+
+	imd->auto_refresh_id = gtk_timeout_add((guint32)interval, image_auto_refresh_cb, imd);
+	imd->auto_refresh_interval = interval;
+}
+
 /* allow top window to be resized ? */
 
 void image_top_window_set_sync(ImageWindow *imd, gint allow_sync)
@@ -2256,7 +2559,7 @@ void image_background_set_black(ImageWindow *imd, gint black)
 	if (!imd) return;
 
 	style = gtk_style_copy(gtk_widget_get_style(imd->widget));
-	gtk_style_ref(style);
+	g_object_ref(G_OBJECT(style));
 
 	if (black)
 		{
@@ -2264,7 +2567,7 @@ void image_background_set_black(ImageWindow *imd, gint black)
 		}
 
 	gtk_widget_set_style(imd->image, style);
-	gtk_style_unref(style);
+	g_object_unref(G_OBJECT(style));
 
 	if (GTK_WIDGET_VISIBLE(imd->widget)) image_border_clear(imd);
 }
@@ -2276,7 +2579,7 @@ void image_background_set_color(ImageWindow *imd, GdkColor *color)
 	if (!imd) return;
 
 	style = gtk_style_copy(gtk_widget_get_style(imd->widget));
-	gtk_style_ref(style);
+	g_object_ref(G_OBJECT(style));
 
 	if (color)
 		{
@@ -2290,7 +2593,7 @@ void image_background_set_color(ImageWindow *imd, GdkColor *color)
 		}
 
 	gtk_widget_set_style(imd->image, style);
-	gtk_style_unref(style);
+	g_object_unref(G_OBJECT(style));
 
 	if (GTK_WIDGET_VISIBLE(imd->widget)) image_border_clear(imd);
 }
@@ -2307,7 +2610,7 @@ void image_to_root_window(ImageWindow *imd, gint scaled)
 	if (!imd || !imd->pixbuf) return;
 
 
-	rootwindow = (GdkWindow *) &gdk_root_parent;	/* hmm, don't know, correct? */
+	rootwindow = GDK_ROOT_PARENT();	/* hmm, don't know, correct? */
 	gdkvisual = gdk_window_get_visual(rootwindow);
 	if (gdkvisual != gdk_visual_get_system()) return;
 
@@ -2323,8 +2626,8 @@ void image_to_root_window(ImageWindow *imd, gint scaled)
 	gdk_pixbuf_render_pixmap_and_mask (pb, &pixmap, NULL, 128);
 	gdk_window_set_back_pixmap(rootwindow, pixmap, FALSE);
 	gdk_window_clear(rootwindow);
-	gdk_pixbuf_unref(pb);
-	gdk_pixmap_unref(pixmap);
+	g_object_unref(pb);
+	g_object_unref(pixmap);
 
 	gdk_flush();
 }
@@ -2339,6 +2642,8 @@ void image_to_root_window(ImageWindow *imd, gint scaled)
 static void image_free(ImageWindow *imd)
 {
 	image_read_ahead_cancel(imd);
+	image_post_buffer_set(imd, NULL, NULL);
+	image_auto_refresh(imd, -1);
 
 	g_free(imd->image_path);
 	g_free(imd->title);
@@ -2346,7 +2651,7 @@ static void image_free(ImageWindow *imd)
 
 	image_reset(imd);
 	image_tile_sync_count(imd, 0);
-	if (imd->pixbuf) gdk_pixbuf_unref(imd->pixbuf);
+	if (imd->pixbuf) g_object_unref(imd->pixbuf);
 
 	g_free(imd);
 }
@@ -2391,10 +2696,24 @@ ImageWindow *image_new(gint frame)
 	imd->read_ahead_pixbuf = NULL;
 	imd->read_ahead_path = NULL;
 
+	imd->completed = FALSE;
+
+	imd->auto_refresh_id = -1;
+	imd->auto_refresh_interval = -1;
+
+	imd->func_update = NULL;
+	imd->func_complete = NULL;
+
+	imd->func_btn1 = NULL;
+	imd->func_btn2 = NULL;
+	imd->func_btn3 = NULL;
+	imd->func_btn4 = NULL;
+	imd->func_btn5 = NULL;	
+
 	imd->eventbox = gtk_event_box_new();
 
-	gtk_signal_connect_after(GTK_OBJECT(imd->eventbox), "size_allocate",
-				 GTK_SIGNAL_FUNC(image_size_cb), imd);
+	g_signal_connect_after(G_OBJECT(imd->eventbox), "size_allocate",
+			       G_CALLBACK(image_size_cb), imd);
 
 	if (imd->has_frame)
 		{
@@ -2404,15 +2723,13 @@ ImageWindow *image_new(gint frame)
 		gtk_widget_show(imd->eventbox);
 
 		GTK_WIDGET_SET_FLAGS(imd->widget, GTK_CAN_FOCUS);
-		gtk_signal_connect(GTK_OBJECT(imd->widget), "focus_in_event",
-				   GTK_SIGNAL_FUNC(image_focus_in_cb), imd);
-		gtk_signal_connect(GTK_OBJECT(imd->widget), "focus_out_event",
-				   GTK_SIGNAL_FUNC(image_focus_out_cb), imd);
+		g_signal_connect(G_OBJECT(imd->widget), "focus_in_event",
+				 G_CALLBACK(image_focus_in_cb), imd);
+		g_signal_connect(G_OBJECT(imd->widget), "focus_out_event",
+				 G_CALLBACK(image_focus_out_cb), imd);
 
-		gtk_signal_connect_after(GTK_OBJECT(imd->widget), "draw",
-					 GTK_SIGNAL_FUNC(image_focus_draw), imd);
-		gtk_signal_connect_after(GTK_OBJECT(imd->widget), "expose_event",
-					 GTK_SIGNAL_FUNC(image_focus_expose), imd);
+		g_signal_connect_after(G_OBJECT(imd->widget), "expose_event",
+				       G_CALLBACK(image_focus_expose), imd);
 		}
 	else
 		{
@@ -2421,24 +2738,27 @@ ImageWindow *image_new(gint frame)
 
 	imd->image = gtk_drawing_area_new();
 
-	gtk_signal_connect(GTK_OBJECT(imd->image), "motion_notify_event",
-			   GTK_SIGNAL_FUNC(image_mouse_motion_cb), imd);
-	gtk_signal_connect(GTK_OBJECT(imd->image), "button_press_event",
-			   GTK_SIGNAL_FUNC(image_mouse_press_cb), imd);
-	gtk_signal_connect(GTK_OBJECT(imd->image), "button_release_event",
-			   GTK_SIGNAL_FUNC(image_mouse_release_cb), imd);
+	g_signal_connect(G_OBJECT(imd->image), "motion_notify_event",
+			 G_CALLBACK(image_mouse_motion_cb), imd);
+	g_signal_connect(G_OBJECT(imd->image), "button_press_event",
+			 G_CALLBACK(image_mouse_press_cb), imd);
+	g_signal_connect(G_OBJECT(imd->image), "button_release_event",
+			 G_CALLBACK(image_mouse_release_cb), imd);
 	gtk_widget_set_events(imd->image, GDK_POINTER_MOTION_MASK | GDK_BUTTON_RELEASE_MASK | GDK_BUTTON_PRESS_MASK);
 
-	gtk_signal_connect(GTK_OBJECT(imd->image), "expose_event",
-			   GTK_SIGNAL_FUNC(image_expose_cb), imd);
-	gtk_signal_connect(GTK_OBJECT(imd->image), "drag_begin",
-			   GTK_SIGNAL_FUNC(image_mouse_drag_cb), imd);
+	g_signal_connect(G_OBJECT(imd->image), "expose_event",
+			 G_CALLBACK(image_expose_cb), imd);
+	g_signal_connect(G_OBJECT(imd->image), "drag_begin",
+			 G_CALLBACK(image_mouse_drag_cb), imd);
+
+	g_signal_connect(G_OBJECT(imd->image), "scroll_event",
+			 G_CALLBACK(image_scroll_cb), imd);
 
 	gtk_container_add(GTK_CONTAINER(imd->eventbox), imd->image);
 	gtk_widget_show(imd->image);
 
-	gtk_signal_connect(GTK_OBJECT(imd->widget), "destroy",
-			   GTK_SIGNAL_FUNC(image_destroy_cb), imd);
+	g_signal_connect(G_OBJECT(imd->widget), "destroy",
+			 G_CALLBACK(image_destroy_cb), imd);
 
 	return imd;
 }
