@@ -1,6 +1,6 @@
 /*
  * GQview
- * (C) 2002 John Ellis
+ * (C) 2004 John Ellis
  *
  * Author: John Ellis
  *
@@ -20,11 +20,14 @@ ImageWindow *image_new(gint frame);
 void image_attach_window(ImageWindow *imd, GtkWidget *window,
 			 const gchar *title, const gchar *title_right, gint show_zoom);
 void image_set_update_func(ImageWindow *imd,
-			   void (*update_func)(ImageWindow *imd, gpointer data),
+			   void (*func)(ImageWindow *imd, gpointer data),
 			   gpointer data);
 void image_set_button_func(ImageWindow *imd, gint button,
-			   void (*func)(ImageWindow *, GdkEventButton *, gpointer),
+			   void (*func)(ImageWindow *, guint32 time, gdouble x, gdouble y, guint state, gpointer),
 			   gpointer data);
+void image_set_complete_func(ImageWindow *imd,
+			     void (*func)(ImageWindow *, gint preload, gpointer),
+			     gpointer data);
 
 /* path, name */
 const gchar *image_get_path(ImageWindow *imd);
@@ -57,6 +60,9 @@ gfloat image_zoom_get_default(ImageWindow *imd, gint mode);
 /* read ahead, pass NULL to cancel */
 void image_prebuffer_set(ImageWindow *imd, const gchar *path);
 
+/* auto refresh, interval is 1/1000 sec, 0 uses default, -1 disables */
+void image_auto_refresh(ImageWindow *imd, gint interval);
+
 /* allow top window to be resized ? */
 void image_top_window_set_sync(ImageWindow *imd, gint allow_sync);
 
