@@ -1205,9 +1205,9 @@ static GtkWidget *view_confirm_dir_list(ViewWindow *vw, GList *list)
 
 	menu_item_add_stock(menu, _("Dropped list includes directories."), GTK_STOCK_DND_MULTIPLE, NULL, NULL);
 	menu_item_add_divider(menu);
-	menu_item_add_stock(menu, _("Add contents"), GTK_STOCK_OK, G_CALLBACK(view_dir_list_add), d);
-	menu_item_add_stock(menu, _("Add contents recursive"), GTK_STOCK_ADD, G_CALLBACK(view_dir_list_recurse), d);
-	menu_item_add_stock(menu, _("Skip directories"), GTK_STOCK_REMOVE, G_CALLBACK(view_dir_list_skip), d);
+	menu_item_add_stock(menu, _("_Add contents"), GTK_STOCK_OK, G_CALLBACK(view_dir_list_add), d);
+	menu_item_add_stock(menu, _("Add contents _recursive"), GTK_STOCK_ADD, G_CALLBACK(view_dir_list_recurse), d);
+	menu_item_add_stock(menu, _("_Skip directories"), GTK_STOCK_REMOVE, G_CALLBACK(view_dir_list_skip), d);
 	menu_item_add_divider(menu);
 	menu_item_add_stock(menu, _("Cancel"), GTK_STOCK_CANCEL, G_CALLBACK(view_dir_list_cancel), d);
 
