@@ -1,6 +1,6 @@
 /*
  * GQview
- * (C) 2002 John Ellis
+ * (C) 2004 John Ellis
  *
  * Author: John Ellis
  *
@@ -21,6 +21,7 @@
 #include "view_dir_tree.h"
 #include "view_file_list.h"
 #include "view_file_icon.h"
+#include "ui_bookmark.h"
 #include "ui_fileops.h"
 #include "ui_menu.h"
 #include "ui_tabcomp.h"
@@ -33,9 +34,10 @@
 #define MAIN_WINDOW_DIV_HPOS -1
 #define MAIN_WINDOW_DIV_VPOS 200
 
-#define TOOLWINDOW_DEF_WIDTH 224
+#define TOOLWINDOW_DEF_WIDTH 260
 #define TOOLWINDOW_DEF_HEIGHT 450
 #define PROGRESS_WIDTH 150
+#define ZOOM_LABEL_WIDTH 64
 
 #define PANE_DIVIDER_SIZE 10
 
@@ -114,10 +116,11 @@ static void layout_path_entry_tab_cb(const gchar *path, gpointer data)
 		{
 		if ((!lw->path || strcmp(lw->path, buf) != 0) && layout_set_path(lw, buf))
 			{
+			gint pos = -1;
 			/* put the '/' back, if we are in tab completion for a dir and result was path change */
-			gtk_entry_append_text(GTK_ENTRY(lw->path_entry), "/");
-			gtk_entry_set_position(GTK_ENTRY(lw->path_entry),
-					       strlen(gtk_entry_get_text(GTK_ENTRY(lw->path_entry))));
+			gtk_editable_insert_text(GTK_EDITABLE(lw->path_entry), "/", -1, &pos);
+			gtk_editable_set_position(GTK_EDITABLE(lw->path_entry),
+						  strlen(gtk_entry_get_text(GTK_ENTRY(lw->path_entry))));
 			}
 		}
 	else if (lw->path && strcmp(lw->path, base) == 0)
@@ -179,10 +182,9 @@ static GtkWidget *layout_tool_setup(LayoutWindow *lw)
 	gtk_box_pack_start(GTK_BOX(box), tabcomp, FALSE, FALSE, 0);
 	gtk_widget_show(tabcomp);
 
-	/* connect to hide signal of popup of the combo list, after hide it will sync the path */
 	popwin = gtk_widget_get_toplevel(GTK_COMBO(tabcomp)->list);
-	gtk_signal_connect_after(GTK_OBJECT(popwin), "hide",
-				 GTK_SIGNAL_FUNC(layout_path_entry_changed_cb), lw);
+	g_signal_connect_after(G_OBJECT(popwin), "hide",
+			       G_CALLBACK(layout_path_entry_changed_cb), lw);
 
 	if (lw->tree_view)
 		{
@@ -245,16 +247,34 @@ static void layout_sort_button_press_cb(GtkWidget *widget, gpointer data)
 {
 	LayoutWindow *lw = data;
 	GtkWidget *menu;
+	GdkEvent *event;
+	guint32 etime;
 
-	menu = submenu_add_sort(NULL, GTK_SIGNAL_FUNC(layout_sort_menu_cb), lw, FALSE, FALSE, TRUE, lw->sort_method);
+	menu = submenu_add_sort(NULL, G_CALLBACK(layout_sort_menu_cb), lw, FALSE, FALSE, TRUE, lw->sort_method);
+
+	/* apparently the menu is never sunk, even on a popup */
+	g_object_ref(G_OBJECT(menu));
+	gtk_object_sink(GTK_OBJECT(menu));
 
         /* ascending option */
 	menu_item_add_divider(menu);
-	menu_item_add_check(menu, _("Ascending"), lw->sort_ascend, GTK_SIGNAL_FUNC(layout_sort_menu_ascend_cb), lw);
+	menu_item_add_check(menu, _("Ascending"), lw->sort_ascend, G_CALLBACK(layout_sort_menu_ascend_cb), lw);
 
-	gtk_signal_connect(GTK_OBJECT(menu), "selection_done",
-			   GTK_SIGNAL_FUNC(layout_sort_menu_hide_cb), NULL);
-	gtk_menu_popup(GTK_MENU(menu), NULL, NULL, NULL, NULL, 0, gdk_time_get());
+	g_signal_connect(G_OBJECT(menu), "selection_done",
+			 G_CALLBACK(layout_sort_menu_hide_cb), NULL);
+
+	event = gtk_get_current_event();
+	if (event)
+		{
+		etime = gdk_event_get_time(event);
+		gdk_event_free(event);
+		}
+	else
+		{
+		etime = 0;
+		}
+
+	gtk_menu_popup(GTK_MENU(menu), NULL, NULL, NULL, NULL, 0, etime);
 }
 
 static GtkWidget *layout_sort_button(LayoutWindow *lw)
@@ -262,8 +282,8 @@ static GtkWidget *layout_sort_button(LayoutWindow *lw)
 	GtkWidget *button;
 
 	button = gtk_button_new_with_label(sort_type_get_text(lw->sort_method));
-        gtk_signal_connect(GTK_OBJECT(button), "clicked",
-			   GTK_SIGNAL_FUNC(layout_sort_button_press_cb), lw);
+        g_signal_connect(G_OBJECT(button), "clicked",
+			 G_CALLBACK(layout_sort_button_press_cb), lw);
         gtk_button_set_relief(GTK_BUTTON(button), GTK_RELIEF_NONE);
 
         return button;
@@ -280,8 +300,8 @@ void layout_status_update_progress(LayoutWindow *lw, gfloat val, const gchar *te
 	if (!layout_valid(&lw)) return;
 	if (!lw->info_progress_bar) return;
 
-	gtk_progress_bar_update(GTK_PROGRESS_BAR(lw->info_progress_bar), val);
-	gtk_progress_set_format_string(GTK_PROGRESS(lw->info_progress_bar), text ? text : "");
+	gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(lw->info_progress_bar), val);
+	gtk_progress_bar_set_text(GTK_PROGRESS_BAR(lw->info_progress_bar), text);
 }
 
 void layout_status_update_info(LayoutWindow *lw, const gchar *text)
@@ -293,9 +313,9 @@ void layout_status_update_info(LayoutWindow *lw, const gchar *text)
 	if (!text)
 		{
 		gint n;
-		gint n_bytes = 0;
+		gint64 n_bytes = 0;
 		gint s;
-		gint s_bytes = 0;
+		gint64 s_bytes = 0;
 		const gchar *ss;
 		gchar *b;
 		gchar *sb;
@@ -360,7 +380,7 @@ void layout_status_update_image(LayoutWindow *lw)
 
 	if (lw->image->unknown)
 		{
-		if (image_get_path(lw->image) && access(image_get_path(lw->image), R_OK) < 0)
+		if (image_get_path(lw->image) && !access_file(image_get_path(lw->image), R_OK))
 			{
 			text = g_strdup_printf(_("(no read permission) %s bytes"), b);
 			}
@@ -394,7 +414,7 @@ static GtkWidget *layout_status_label(gchar *text, GtkWidget *box, gint start, g
 	GtkWidget *frame;
 
 	frame = gtk_frame_new(NULL);
-	if (size) gtk_widget_set_usize (frame, size, -1);
+	if (size) gtk_widget_set_size_request(frame, size, -1);
 	gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_IN);
 	if (start)
 		{
@@ -441,8 +461,7 @@ static void layout_status_setup(LayoutWindow *lw, GtkWidget *box, gint small_for
 		hbox = lw->info_box;
 		}
 	lw->info_progress_bar = gtk_progress_bar_new();
-	gtk_progress_set_show_text(GTK_PROGRESS(lw->info_progress_bar), TRUE);
-	gtk_widget_set_usize(lw->info_progress_bar, PROGRESS_WIDTH, -1);
+	gtk_widget_set_size_request(lw->info_progress_bar, PROGRESS_WIDTH, -1);
 	gtk_box_pack_start(GTK_BOX(hbox), lw->info_progress_bar, FALSE, FALSE, 0);
 	gtk_widget_show(lw->info_progress_bar);
 
@@ -463,7 +482,7 @@ static void layout_status_setup(LayoutWindow *lw, GtkWidget *box, gint small_for
 		hbox = lw->info_box;
 		}
 	lw->info_details = layout_status_label(NULL, hbox, TRUE, 0, TRUE);
-	lw->info_zoom = layout_status_label(NULL, hbox, FALSE, 48, FALSE);
+	lw->info_zoom = layout_status_label(NULL, hbox, FALSE, ZOOM_LABEL_WIDTH, FALSE);
 }
 
 /*
@@ -552,7 +571,7 @@ GList *layout_list(LayoutWindow *lw)
 	return NULL;
 }
 
-gint layout_list_count(LayoutWindow *lw, gint *bytes)
+gint layout_list_count(LayoutWindow *lw, gint64 *bytes)
 {
 	if (!layout_valid(&lw)) return 0;
 
@@ -602,6 +621,15 @@ GList *layout_selection_list(LayoutWindow *lw)
 {
 	if (!layout_valid(&lw)) return NULL;
 
+	if (layout_image_get_collection(lw, NULL))
+		{
+		const gchar *path;
+
+		path = layout_image_get_path(lw);
+		if (path) return g_list_append(NULL, g_strdup(path));
+		return NULL;
+		}
+
 	if (lw->vfl) return vflist_selection_get_list(lw->vfl);
 	if (lw->vfi) return vficon_selection_get_list(lw->vfi);
 
@@ -618,7 +646,7 @@ GList *layout_selection_list_by_index(LayoutWindow *lw)
 	return NULL;
 }
 
-gint layout_selection_count(LayoutWindow *lw, gint *bytes)
+gint layout_selection_count(LayoutWindow *lw, gint64 *bytes)
 {
 	if (!layout_valid(&lw)) return 0;
 
@@ -770,6 +798,8 @@ static gint layout_check_for_update_cb(gpointer data)
 {
 	LayoutWindow *lw = data;
 
+	if (!update_on_time_change) return TRUE;
+
 	if (lw->path)
 		{
 		time_t new_time;
@@ -790,6 +820,8 @@ void layout_thumb_set(LayoutWindow *lw, gint enable)
 {
 	if (!layout_valid(&lw)) return;
 
+	if (lw->thumbs_enabled == enable) return;
+
 	lw->thumbs_enabled = enable;
 
 	layout_util_sync_thumb(lw);
@@ -1025,13 +1057,24 @@ static void layout_tools_setup(LayoutWindow *lw, GtkWidget *tools, GtkWidget *fi
 
 	if (!lw->tools)
 		{
+		GdkGeometry geometry;
+
 		lw->tools = gtk_window_new(GTK_WINDOW_TOPLEVEL);
-		gtk_signal_connect(GTK_OBJECT(lw->tools), "delete_event",
-				   GTK_SIGNAL_FUNC(layout_tools_delete_cb), lw);
-	        gtk_window_set_policy(GTK_WINDOW(lw->tools), TRUE, TRUE, FALSE);
+		g_signal_connect(G_OBJECT(lw->tools), "delete_event",
+				 G_CALLBACK(layout_tools_delete_cb), lw);
+
+		geometry.min_width = 32;
+		geometry.min_height = 32;
+		geometry.base_width = TOOLWINDOW_DEF_WIDTH;
+		geometry.base_height = TOOLWINDOW_DEF_HEIGHT;
+		gtk_window_set_geometry_hints(GTK_WINDOW(lw->tools), NULL, &geometry,
+					      GDK_HINT_MIN_SIZE | GDK_HINT_BASE_SIZE);
+
+
+	        gtk_window_set_resizable(GTK_WINDOW(lw->tools), TRUE);
         	gtk_window_set_title(GTK_WINDOW(lw->tools), _("GQview Tools"));
         	gtk_window_set_wmclass(GTK_WINDOW(lw->tools), "tools", "GQview");
-        	gtk_container_border_width(GTK_CONTAINER(lw->tools), 0);
+        	gtk_container_set_border_width(GTK_CONTAINER(lw->tools), 0);
 	        gtk_window_add_accel_group(GTK_WINDOW(lw->tools), lw->accel_grp);
 
 		window_set_icon(lw->tools, (const gchar **)tools_xpm, NULL);
@@ -1059,8 +1102,6 @@ static void layout_tools_setup(LayoutWindow *lw, GtkWidget *tools, GtkWidget *fi
 		{
 		lw->tools_pane = gtk_hpaned_new();
 		}
-	gtk_paned_handle_size(GTK_PANED(lw->tools_pane), PANE_DIVIDER_SIZE);
-	gtk_paned_gutter_size(GTK_PANED(lw->tools_pane), PANE_DIVIDER_SIZE);
 	gtk_box_pack_start(GTK_BOX(vbox), lw->tools_pane, TRUE, TRUE, 0);
 	gtk_widget_show(lw->tools_pane);
 
@@ -1075,7 +1116,7 @@ static void layout_tools_setup(LayoutWindow *lw, GtkWidget *tools, GtkWidget *fi
 		if (save_window_positions)
 			{
 			gtk_window_set_default_size(GTK_WINDOW(lw->tools), float_window_w, float_window_h);
-			gtk_widget_set_uposition(lw->tools, float_window_x, float_window_y);
+			gtk_window_move(GTK_WINDOW(lw->tools), float_window_x, float_window_y);
         	        }
 		else
 			{
@@ -1182,6 +1223,8 @@ static void layout_grid_setup(LayoutWindow *lw)
 	tools = layout_tools_new(lw);
 	files = layout_list_new(lw);
 
+	image = layout_bars_prepare(lw, image);
+
 	if (lw->tools_float || lw->tools_hidden)
 		{
 		gtk_box_pack_start(GTK_BOX(lw->group_box), image, TRUE, TRUE, 0);
@@ -1189,6 +1232,8 @@ static void layout_grid_setup(LayoutWindow *lw)
 
 		layout_tools_setup(lw, tools, files);
 
+		gtk_widget_grab_focus(lw->image->widget);
+
 		return;
 		}
 	else if (lw->tools)
@@ -1204,12 +1249,8 @@ static void layout_grid_setup(LayoutWindow *lw)
 	layout_grid_compute(lw, image, tools, files, &w1, &w2, &w3);
 
 	v = lw->v_pane = gtk_vpaned_new();
-	gtk_paned_handle_size(GTK_PANED(v), PANE_DIVIDER_SIZE);
-        gtk_paned_gutter_size(GTK_PANED(v), PANE_DIVIDER_SIZE);
 
 	h = lw->h_pane = gtk_hpaned_new();
-	gtk_paned_handle_size(GTK_PANED(h), PANE_DIVIDER_SIZE);
-        gtk_paned_gutter_size(GTK_PANED(h), PANE_DIVIDER_SIZE);
 
 	if (!layout_location_vertical(priority_location))
 		{
@@ -1248,6 +1289,8 @@ static void layout_grid_setup(LayoutWindow *lw)
 
 	gtk_paned_set_position(GTK_PANED(lw->h_pane), lw->div_h);
 	gtk_paned_set_position(GTK_PANED(lw->v_pane), lw->div_v);
+
+	gtk_widget_grab_focus(lw->image->widget);
 }
 
 void layout_style_set(LayoutWindow *lw, gint style, const gchar *order)
@@ -1281,6 +1324,7 @@ void layout_style_set(LayoutWindow *lw, gint style, const gchar *order)
 	lw->path = NULL;
 	old_image = lw->image;
 	lw->image = NULL;
+	lw->utility_box = NULL;
 
 	layout_geometry_get_dividers(lw, &lw->div_h, &lw->div_v);
 
@@ -1311,12 +1355,12 @@ void layout_style_set(LayoutWindow *lw, gint style, const gchar *order)
 	lw->info_details = NULL;
 	lw->info_zoom = NULL;
 
+	if (lw->menu_fact) g_object_unref(G_OBJECT(lw->menu_fact));
+	lw->menu_fact = NULL;
+
 	gtk_container_remove(GTK_CONTAINER(lw->main_box), lw->group_box);
 	lw->group_box = NULL;
 
-	if (lw->menu_fact) gtk_object_unref(GTK_OBJECT(lw->menu_fact));
-	lw->menu_fact = NULL;
-
 	/* re-fill */
 
 	layout_grid_setup(lw);
@@ -1496,8 +1540,10 @@ void layout_free(LayoutWindow *lw)
 		gtk_timeout_remove(lw->last_time_id);
 		}
 
-	if (lw->menu_fact) gtk_object_unref(GTK_OBJECT(lw->menu_fact));
-	if (lw->tooltips) gtk_object_unref(GTK_OBJECT(lw->tooltips));
+	layout_bars_close(lw);
+
+	if (lw->menu_fact) g_object_unref(G_OBJECT(lw->menu_fact));
+	if (lw->tooltips) g_object_unref(G_OBJECT(lw->tooltips));
 	gtk_widget_destroy(lw->window);
 
 	g_free(lw->path);
@@ -1516,6 +1562,7 @@ static gint layout_delete_cb(GtkWidget *widget, GdkEventAny *event, gpointer dat
 LayoutWindow *layout_new(const gchar *path, gint popped, gint hidden)
 {
 	LayoutWindow *lw;
+	GdkGeometry geometry;
 
 	lw = g_new0(LayoutWindow, 1);
 
@@ -1528,6 +1575,14 @@ LayoutWindow *layout_new(const gchar *path, gint popped, gint hidden)
 
 	lw->toolbar_hidden = toolbar_hidden;
 
+	lw->utility_box = NULL;
+	lw->bar_sort = NULL;
+	lw->bar_sort_enabled = FALSE;
+	lw->bar_exif = NULL;
+	lw->bar_exif_enabled = FALSE;
+	lw->bar_exif_size = -1;
+	lw->bar_exif_advanced = FALSE;
+
 	/* default layout */
 
 	layout_config_parse(layout_style, layout_order,
@@ -1553,20 +1608,27 @@ LayoutWindow *layout_new(const gchar *path, gint popped, gint hidden)
 	/* window */
 
 	lw->window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
-	gtk_window_set_policy(GTK_WINDOW(lw->window), TRUE, TRUE, FALSE);
+	gtk_window_set_resizable(GTK_WINDOW(lw->window), TRUE);
 
 	gtk_window_set_title(GTK_WINDOW(lw->window), "GQview");
 	gtk_window_set_wmclass(GTK_WINDOW(lw->window), "gqview", "GQview");
-	gtk_container_border_width(GTK_CONTAINER(lw->window), 0);
+	gtk_container_set_border_width(GTK_CONTAINER(lw->window), 0);
 
 	window_set_icon(lw->window, NULL, NULL);
 
+	geometry.min_width = 32;
+	geometry.min_height = 32;
+	geometry.base_width = MAINWINDOW_DEF_WIDTH;
+	geometry.base_height = MAINWINDOW_DEF_HEIGHT;
+	gtk_window_set_geometry_hints(GTK_WINDOW(lw->window), NULL, &geometry,
+				      GDK_HINT_MIN_SIZE | GDK_HINT_BASE_SIZE);
+
 	if (save_window_positions)
 		{
 		gtk_window_set_default_size(GTK_WINDOW(lw->window), main_window_w, main_window_h);
 		if (!layout_window_list)
 			{
-			gtk_widget_set_uposition(lw->window, main_window_x, main_window_y);
+			gtk_window_move(GTK_WINDOW(lw->window), main_window_x, main_window_y);
 			}
 		}
 	else
@@ -1574,17 +1636,21 @@ LayoutWindow *layout_new(const gchar *path, gint popped, gint hidden)
 		gtk_window_set_default_size(GTK_WINDOW(lw->window), MAINWINDOW_DEF_WIDTH, MAINWINDOW_DEF_HEIGHT);
 		}
 
-	gtk_signal_connect(GTK_OBJECT(lw->window), "delete_event",
-			   GTK_SIGNAL_FUNC(layout_delete_cb), lw);
+	g_signal_connect(G_OBJECT(lw->window), "delete_event",
+			 G_CALLBACK(layout_delete_cb), lw);
 
 	layout_keyboard_init(lw);
 
+#if 0
 	gtk_widget_realize(lw->window);
+#endif
 
 	lw->accel_grp = gtk_accel_group_new();
 	gtk_window_add_accel_group(GTK_WINDOW(lw->window), lw->accel_grp);
 
 	lw->tooltips = gtk_tooltips_new();
+	g_object_ref(G_OBJECT(lw->tooltips));
+	gtk_object_sink(GTK_OBJECT(lw->tooltips));
 
 	lw->main_box = gtk_vbox_new(FALSE, 0);
 	gtk_container_add(GTK_CONTAINER(lw->window), lw->main_box);
@@ -1638,32 +1704,38 @@ static void layout_real_time_update(LayoutWindow *lw)
 
 static void layout_real_renamed(LayoutWindow *lw, const gchar *source, const gchar *dest)
 {
+	gint update = FALSE;
+
 	if (lw->image) layout_image_maint_renamed(lw, source, dest);
 
-	if (lw->vfl) vflist_maint_renamed(lw->vfl, source, dest);
-	if (lw->vfi) vficon_maint_renamed(lw->vfi, source, dest);
+	if (lw->vfl) update |= vflist_maint_renamed(lw->vfl, source, dest);
+	if (lw->vfi) update |= vficon_maint_renamed(lw->vfi, source, dest);
 
-	layout_real_time_update(lw);
+	if (update) layout_real_time_update(lw);
 }
 
 static void layout_real_removed(LayoutWindow *lw, const gchar *path, GList *ignore_list)
 {
+	gint update = FALSE;
+
 	if (lw->image) layout_image_maint_removed(lw, path);
 
-	if (lw->vfl) vflist_maint_removed(lw->vfl, path, ignore_list);
-	if (lw->vfi) vficon_maint_removed(lw->vfi, path, ignore_list);
+	if (lw->vfl) update |= vflist_maint_removed(lw->vfl, path, ignore_list);
+	if (lw->vfi) update |= vficon_maint_removed(lw->vfi, path, ignore_list);
 
-	layout_real_time_update(lw);
+	if (update) layout_real_time_update(lw);
 }
 
 static void layout_real_moved(LayoutWindow *lw, const gchar *source, const gchar *dest, GList *ignore_list)
 {
+	gint update = FALSE;
+
 	if (lw->image) layout_image_maint_moved(lw, source, dest);
 
-	if (lw->vfl) vflist_maint_moved(lw->vfl, source, dest, ignore_list);
-	if (lw->vfi) vficon_maint_moved(lw->vfi, source, dest, ignore_list);
+	if (lw->vfl) update |= vflist_maint_moved(lw->vfl, source, dest, ignore_list);
+	if (lw->vfi) update |= vficon_maint_moved(lw->vfi, source, dest, ignore_list);
 
-	layout_real_time_update(lw);
+	if (update) layout_real_time_update(lw);
 }
 
 void layout_maint_renamed(const gchar *source, const gchar *dest)
