@@ -321,6 +321,7 @@ GtkWidget *layout_config_new(void)
 	gtk_widget_show(hbox);
 
 	scrolled = gtk_scrolled_window_new(NULL, NULL);
+	gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(scrolled), GTK_SHADOW_IN);
 	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolled),
 				       GTK_POLICY_NEVER, GTK_POLICY_NEVER);
 	gtk_box_pack_start(GTK_BOX(lc->box), scrolled, FALSE, FALSE, 0);
@@ -330,6 +331,7 @@ GtkWidget *layout_config_new(void)
 	lc->listview = gtk_tree_view_new_with_model(GTK_TREE_MODEL(store));
 	g_object_unref(store);
 
+	gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(lc->listview), FALSE);
 	gtk_tree_view_set_reorderable(GTK_TREE_VIEW(lc->listview), TRUE);
 
 	column = gtk_tree_view_column_new();
