@@ -1,6 +1,6 @@
 /*
  * GQview
- * (C) 2002 John Ellis
+ * (C) 2004 John Ellis
  *
  * Author: John Ellis
  *
@@ -14,20 +14,24 @@
 
 #include "collect.h"
 #include "dnd.h"
+#include "editors.h"
 #include "fullscreen.h"
 #include "image.h"
 #include "img-view.h"
 #include "info.h"
 #include "layout.h"
+#include "layout_util.h"
 #include "menu.h"
 #include "utilops.h"
 #include "slideshow.h"
+#include "ui_bookmark.h"
 #include "ui_fileops.h"
 #include "ui_menu.h"
 
 #include <gdk/gdkkeysyms.h> /* for keyboard values */
 
 
+static GtkWidget *layout_image_pop_menu(LayoutWindow *lw);
 static void layout_image_set_buttons(LayoutWindow *lw);
 
 
@@ -37,6 +41,42 @@ static void layout_image_set_buttons(LayoutWindow *lw);
  *----------------------------------------------------------------------------
  */
 
+static void layout_image_full_screen_regrab_cb(GtkWidget *widget, gpointer data)
+{
+	LayoutWindow *lw = data;
+
+	/* full screen, re-grab keyboard after popup-menu close */
+	if (lw->full_screen)
+		{
+		gdk_keyboard_grab(lw->full_screen->window->window, TRUE, GDK_CURRENT_TIME);
+		}
+}
+
+
+static void layout_image_fullscreen_menu_pos_cb(GtkMenu *menu, gint *x, gint *y, gboolean *push_in, gpointer data)
+{
+	LayoutWindow *lw = data;
+
+	if (!lw->full_screen) return;
+
+	gdk_window_get_origin(lw->full_screen->imd->image->window, x, y);
+	popup_menu_position_clamp(menu, x, y, 0);
+}
+
+static void layout_image_full_screen_menu_popup(LayoutWindow *lw)
+{
+	GtkWidget *menu;
+
+	menu = layout_image_pop_menu(lw);
+	gtk_menu_popup(GTK_MENU(menu), NULL, NULL, layout_image_fullscreen_menu_pos_cb, lw, 0, GDK_CURRENT_TIME);
+	if (lw->full_screen)
+		{ 
+		/* full screen, re-grab keyboard after popup-menu close */
+		g_signal_connect(G_OBJECT(menu), "selection_done",
+				 G_CALLBACK(layout_image_full_screen_regrab_cb), lw);
+		}
+}
+
 static gint layout_image_full_screen_key_press_cb(GtkWidget *widget, GdkEventKey *event, gpointer data)
 {
 	LayoutWindow *lw = data;
@@ -102,6 +142,7 @@ static gint layout_image_full_screen_key_press_cb(GtkWidget *widget, GdkEventKey
 			break;
 		case GDK_Page_Down: case GDK_KP_Page_Down:
 		case GDK_space:
+		case 'N': case 'n':
 			layout_image_next(lw);
 			stop_signal = TRUE;
 			break;
@@ -133,19 +174,21 @@ static gint layout_image_full_screen_key_press_cb(GtkWidget *widget, GdkEventKey
 			layout_image_full_screen_stop(lw);
 			stop_signal = TRUE;
 			break;
-		case 'Q': case 'q':
-			exit_gqview();
-			return FALSE;
-                        break;
 		case 'S': case 's':
 			layout_image_slideshow_toggle(lw);
 			break;
 		case 'P': case 'p':
 			layout_image_slideshow_pause_toggle(lw);
 			break;
+		case 'F': case 'f':
 		case 'V': case 'v':
 			layout_image_full_screen_stop(lw);
 			break;
+		case GDK_Menu:
+		case GDK_F10:
+			layout_image_full_screen_menu_popup(lw);
+			stop_signal = TRUE;
+			break;
 		}
 
 	if (event->state & GDK_CONTROL_MASK)
@@ -177,6 +220,12 @@ static gint layout_image_full_screen_key_press_cb(GtkWidget *widget, GdkEventKey
 			case '8':
 				n = 7;
 				break;
+			case '9':
+				n = 8;
+				break;
+			case '0':
+				n = 9;
+				break;
 			case 'C': case 'c':
 				layout_image_full_screen_stop(lw);
 				file_util_copy(layout_image_get_path(lw), NULL, NULL);
@@ -193,6 +242,14 @@ static gint layout_image_full_screen_key_press_cb(GtkWidget *widget, GdkEventKey
 				if (confirm_delete) layout_image_full_screen_stop(lw);
 				file_util_delete(layout_image_get_path(lw), NULL);
 				break;
+			case 'P': case 'p':
+				layout_image_full_screen_stop(lw);
+				info_window_new(layout_image_get_path(lw), NULL);
+				break;
+			case 'Q': case 'q':
+				exit_gqview();
+				return FALSE;
+				break;
 			}
 		if (n != -1)
 			{
@@ -266,11 +323,13 @@ void layout_image_full_screen_start(LayoutWindow *lw)
 
 	layout_image_set_buttons(lw);
 
-	gtk_signal_connect(GTK_OBJECT(lw->full_screen->window), "key_press_event",
-			   GTK_SIGNAL_FUNC(layout_image_full_screen_key_press_cb), lw);
+	g_signal_connect(G_OBJECT(lw->full_screen->window), "key_press_event",
+			 G_CALLBACK(layout_image_full_screen_key_press_cb), lw);
 
+#if 0
 	gtk_widget_set_sensitive(lw->window, FALSE);
 	if (lw->tools) gtk_widget_set_sensitive(lw->tools, FALSE);
+#endif
 }
 
 void layout_image_full_screen_stop(LayoutWindow *lw)
@@ -280,8 +339,10 @@ void layout_image_full_screen_stop(LayoutWindow *lw)
 
 	fullscreen_stop(lw->full_screen);
 
+#if 0
 	gtk_widget_set_sensitive(lw->window, TRUE);
 	if (lw->tools) gtk_widget_set_sensitive(lw->tools, TRUE);
+#endif
 }
 
 void layout_image_full_screen_toggle(LayoutWindow *lw)
@@ -502,6 +563,7 @@ static void li_pop_menu_info_cb(GtkWidget *widget, gpointer data)
 {
 	LayoutWindow *lw = data;
 
+	layout_image_full_screen_stop(lw);
 	info_window_new(layout_image_get_path(lw), NULL);
 }
 
@@ -592,77 +654,93 @@ static GtkWidget *layout_image_pop_menu(LayoutWindow *lw)
 
 	menu = popup_menu_short_lived();
 
-	menu_item_add(menu, _("Zoom in"), GTK_SIGNAL_FUNC(li_pop_menu_zoom_in_cb), lw);
-	menu_item_add(menu, _("Zoom out"), GTK_SIGNAL_FUNC(li_pop_menu_zoom_out_cb), lw);
-	menu_item_add(menu, _("Zoom 1:1"), GTK_SIGNAL_FUNC(li_pop_menu_zoom_1_1_cb), lw);
-	menu_item_add(menu, _("Fit image to window"), GTK_SIGNAL_FUNC(li_pop_menu_zoom_fit_cb), lw);
+	menu_item_add_stock(menu, _("Zoom _in"), GTK_STOCK_ZOOM_IN, G_CALLBACK(li_pop_menu_zoom_in_cb), lw);
+	menu_item_add_stock(menu, _("Zoom _out"), GTK_STOCK_ZOOM_OUT, G_CALLBACK(li_pop_menu_zoom_out_cb), lw);
+	menu_item_add_stock(menu, _("Zoom _1:1"), GTK_STOCK_ZOOM_100, G_CALLBACK(li_pop_menu_zoom_1_1_cb), lw);
+	menu_item_add_stock(menu, _("Fit image to _window"), GTK_STOCK_ZOOM_FIT, G_CALLBACK(li_pop_menu_zoom_fit_cb), lw);
 	menu_item_add_divider(menu);
 
-	submenu = submenu_add_edit(menu, &item, GTK_SIGNAL_FUNC(li_pop_menu_edit_cb), lw);
+	submenu = submenu_add_edit(menu, &item, G_CALLBACK(li_pop_menu_edit_cb), lw);
 	if (!path) gtk_widget_set_sensitive(item, FALSE);
 	menu_item_add_divider(submenu);
-	menu_item_add(submenu, _("Set as wallpaper"), GTK_SIGNAL_FUNC(li_pop_menu_wallpaper_cb), lw);
+	menu_item_add(submenu, _("Set as _wallpaper"), G_CALLBACK(li_pop_menu_wallpaper_cb), lw);
 
-	item = submenu_add_alter(menu, GTK_SIGNAL_FUNC(li_pop_menu_alter_cb), lw);
+	item = submenu_add_alter(menu, G_CALLBACK(li_pop_menu_alter_cb), lw);
 
-	item = menu_item_add(menu, _("Properties"), GTK_SIGNAL_FUNC(li_pop_menu_info_cb), lw);
+	item = menu_item_add_stock(menu, _("_Properties"), GTK_STOCK_PROPERTIES, G_CALLBACK(li_pop_menu_info_cb), lw);
 	if (!path) gtk_widget_set_sensitive(item, FALSE);
 
-	item = menu_item_add(menu, _("View in new window"), GTK_SIGNAL_FUNC(li_pop_menu_new_cb), lw);
+	item = menu_item_add_stock(menu, _("View in _new window"), GTK_STOCK_NEW, G_CALLBACK(li_pop_menu_new_cb), lw);
 	if (!path || fullscreen) gtk_widget_set_sensitive(item, FALSE);
 
 	menu_item_add_divider(menu);
 
-	item = menu_item_add(menu, _("Copy..."), GTK_SIGNAL_FUNC(li_pop_menu_copy_cb), lw);
+	item = menu_item_add_stock(menu, _("_Copy..."), GTK_STOCK_COPY, G_CALLBACK(li_pop_menu_copy_cb), lw);
 	if (!path) gtk_widget_set_sensitive(item, FALSE);
-	item = menu_item_add(menu, _("Move..."), GTK_SIGNAL_FUNC(li_pop_menu_move_cb), lw);
+	item = menu_item_add(menu, _("_Move..."), G_CALLBACK(li_pop_menu_move_cb), lw);
 	if (!path) gtk_widget_set_sensitive(item, FALSE);
-	item = menu_item_add(menu, _("Rename..."), GTK_SIGNAL_FUNC(li_pop_menu_rename_cb), lw);
+	item = menu_item_add(menu, _("_Rename..."), G_CALLBACK(li_pop_menu_rename_cb), lw);
 	if (!path) gtk_widget_set_sensitive(item, FALSE);
-	item = menu_item_add(menu, _("Delete..."), GTK_SIGNAL_FUNC(li_pop_menu_delete_cb), lw);
+	item = menu_item_add_stock(menu, _("_Delete..."), GTK_STOCK_DELETE, G_CALLBACK(li_pop_menu_delete_cb), lw);
 	if (!path) gtk_widget_set_sensitive(item, FALSE);
 
 	menu_item_add_divider(menu);
 
 	if (layout_image_slideshow_active(lw))
 		{
-		menu_item_add(menu, _("Stop slideshow"), GTK_SIGNAL_FUNC(li_pop_menu_slide_stop_cb), lw);
+		menu_item_add(menu, _("_Stop slideshow"), G_CALLBACK(li_pop_menu_slide_stop_cb), lw);
 		if (layout_image_slideshow_paused(lw))
 			{
-			item = menu_item_add(menu, _("Continue slideshow"),
-					     GTK_SIGNAL_FUNC(li_pop_menu_slide_pause_cb), lw);
+			item = menu_item_add(menu, _("Continue slides_how"),
+					     G_CALLBACK(li_pop_menu_slide_pause_cb), lw);
 			}
 		else
 			{
-			item = menu_item_add(menu, _("Pause slideshow"),
-					     GTK_SIGNAL_FUNC(li_pop_menu_slide_pause_cb), lw);
+			item = menu_item_add(menu, _("Pause slides_how"),
+					     G_CALLBACK(li_pop_menu_slide_pause_cb), lw);
 			}
 		}
 	else
                 {
-		menu_item_add(menu, _("Start slideshow"), GTK_SIGNAL_FUNC(li_pop_menu_slide_start_cb), lw);
-		item = menu_item_add(menu, _("Pause slideshow"), GTK_SIGNAL_FUNC(li_pop_menu_slide_pause_cb), lw);
+		menu_item_add(menu, _("_Start slideshow"), G_CALLBACK(li_pop_menu_slide_start_cb), lw);
+		item = menu_item_add(menu, _("Pause slides_how"), G_CALLBACK(li_pop_menu_slide_pause_cb), lw);
 		gtk_widget_set_sensitive(item, FALSE);
 		}
 
 	if (!fullscreen)
 		{
-		menu_item_add(menu, _("Full screen"), GTK_SIGNAL_FUNC(li_pop_menu_full_screen_cb), lw);
+		menu_item_add(menu, _("_Full screen"), G_CALLBACK(li_pop_menu_full_screen_cb), lw);
 		}
 	else
 		{
-		menu_item_add(menu, _("Exit full screen"), GTK_SIGNAL_FUNC(li_pop_menu_full_screen_cb), lw);
+		menu_item_add(menu, _("Exit _full screen"), G_CALLBACK(li_pop_menu_full_screen_cb), lw);
 		}
 
 	menu_item_add_divider(menu);
 
-	item = menu_item_add_check(menu, _("Hide file list"), lw->tools_hidden,
-				   GTK_SIGNAL_FUNC(li_pop_menu_hide_cb), lw);
+	item = menu_item_add_check(menu, _("Hide file _list"), lw->tools_hidden,
+				   G_CALLBACK(li_pop_menu_hide_cb), lw);
 	if (fullscreen) gtk_widget_set_sensitive(item, FALSE);
 
 	return menu;
 }
 
+static void layout_image_menu_pos_cb(GtkMenu *menu, gint *x, gint *y, gboolean *push_in, gpointer data)
+{
+	LayoutWindow *lw = data;
+
+	gdk_window_get_origin(lw->image->image->window, x, y);
+	popup_menu_position_clamp(menu, x, y, 0);
+}
+
+void layout_image_menu_popup(LayoutWindow *lw)
+{
+	GtkWidget *menu;
+
+	menu = layout_image_pop_menu(lw);
+	gtk_menu_popup(GTK_MENU(menu), NULL, NULL, layout_image_menu_pos_cb, lw, 0, GDK_CURRENT_TIME);
+}
+
 /*
  *----------------------------------------------------------------------------
  * dnd
@@ -684,7 +762,7 @@ static void layout_image_dnd_receive(GtkWidget *widget, GdkDragContext *context,
 
 		if (info == TARGET_URI_LIST)
 			{
-			list = get_uri_file_list(selection_data->data);
+			list = uri_list_from_text(selection_data->data, TRUE);
 			source = NULL;
 			info_list = NULL;
 			}
@@ -749,19 +827,27 @@ static void layout_image_dnd_get(GtkWidget *widget, GdkDragContext *context,
 	if (path)
 		{
 		gchar *text = NULL;
+		gint len;
+		gint plain_text;
+		GList *list;
+
 		switch (info)
 			{
 			case TARGET_URI_LIST:
-				text = g_strconcat("file:", path, "\r\n", NULL);
+				plain_text = FALSE;
 				break;
 			case TARGET_TEXT_PLAIN:
-				text = g_strdup(path);
+			default:
+				plain_text = TRUE;
 				break;
 			}
+		list = g_list_append(NULL, (gchar *)path);
+		text = uri_text_from_list(list, &len, plain_text);
+		g_list_free(list);
 		if (text)
 			{
 			gtk_selection_data_set (selection_data, selection_data->target,
-						8, text, strlen(text));
+						8, text, len);
 			g_free(text);
 			}
 		}
@@ -804,17 +890,17 @@ static void layout_image_dnd_init(LayoutWindow *lw)
 	gtk_drag_source_set(lw->image->image, GDK_BUTTON2_MASK,
 			    dnd_file_drag_types, dnd_file_drag_types_count,
 			    GDK_ACTION_COPY | GDK_ACTION_MOVE | GDK_ACTION_LINK);
-	gtk_signal_connect(GTK_OBJECT(lw->image->image), "drag_data_get",
-			   GTK_SIGNAL_FUNC(layout_image_dnd_get), lw);
-	gtk_signal_connect(GTK_OBJECT(lw->image->image), "drag_end",
-			   GTK_SIGNAL_FUNC(layout_image_dnd_end), lw);
+	g_signal_connect(G_OBJECT(lw->image->image), "drag_data_get",
+			 G_CALLBACK(layout_image_dnd_get), lw);
+	g_signal_connect(G_OBJECT(lw->image->image), "drag_end",
+			 G_CALLBACK(layout_image_dnd_end), lw);
 
 	gtk_drag_dest_set(lw->image->image,
 			  GTK_DEST_DEFAULT_MOTION | GTK_DEST_DEFAULT_DROP,
 			  dnd_file_drop_types, dnd_file_drop_types_count,
-                          GDK_ACTION_COPY);
-	gtk_signal_connect(GTK_OBJECT(lw->image->image), "drag_data_received",
-			   GTK_SIGNAL_FUNC(layout_image_dnd_receive), lw);
+                          GDK_ACTION_COPY | GDK_ACTION_MOVE | GDK_ACTION_LINK);
+	g_signal_connect(G_OBJECT(lw->image->image), "drag_data_received",
+			 G_CALLBACK(layout_image_dnd_receive), lw);
 }
 
 
@@ -903,6 +989,7 @@ void layout_image_set_path(LayoutWindow *lw, const gchar *path)
 
 	layout_list_sync_path(lw, path);
 	layout_image_slideshow_continue_check(lw);
+	layout_bars_new_image(lw);
 }
 
 void layout_image_set_with_ahead(LayoutWindow *lw, const gchar *path, const gchar *read_ahead_path)
@@ -966,6 +1053,7 @@ static void layout_image_set_collection_real(LayoutWindow *lw, CollectionData *c
 		}
 
 	layout_image_slideshow_continue_check(lw);
+	layout_bars_new_image(lw);
 }
 
 void layout_image_set_collection(LayoutWindow *lw, CollectionData *cd, CollectInfo *info)
@@ -1121,56 +1209,44 @@ void layout_image_last(LayoutWindow *lw)
  *----------------------------------------------------------------------------
  */
 
-static void layout_image_button1_cb(ImageWindow *imd, GdkEventButton *bevent, gpointer data)
+static void layout_image_button1_cb(ImageWindow *imd, guint32 time, gdouble x, gdouble y, guint state, gpointer data)
 {
 	LayoutWindow *lw = data;
 
 	layout_image_next(lw);
 }
 
-static void layout_image_button2_cb(ImageWindow *imd, GdkEventButton *bevent, gpointer data)
+static void layout_image_button2_cb(ImageWindow *imd, guint32 time, gdouble x, gdouble y, guint state, gpointer data)
 {
 	LayoutWindow *lw = data;
 
 	layout_image_prev(lw);
 }
 
-static void layout_image_full_screen_regrab_cb(GtkWidget *widget, gpointer data)
-{
-	LayoutWindow *lw = data;
-
-	/* full screen, re-grab keyboard after popup-menu close */
-	if (lw->full_screen)
-		{
-		gdk_keyboard_grab(lw->full_screen->window->window, TRUE, GDK_CURRENT_TIME);
-		}
-}
-
-static void layout_image_button3_cb(ImageWindow *imd, GdkEventButton *bevent, gpointer data)
+static void layout_image_button3_cb(ImageWindow *imd, guint32 time, gdouble x, gdouble y, guint state, gpointer data)
 {
 	LayoutWindow *lw = data;
 	GtkWidget *menu;
 
 	menu = layout_image_pop_menu(lw);
-	gtk_menu_popup(GTK_MENU(menu), NULL, NULL, NULL, NULL,
-		       bevent->button, bevent->time);
+	gtk_menu_popup(GTK_MENU(menu), NULL, NULL, NULL, NULL, 3, time);
 	if (lw->full_screen)
 		{
 		/* full screen, re-grab keyboard after popup-menu close */
-		gtk_signal_connect(GTK_OBJECT(menu), "selection_done",
-				   GTK_SIGNAL_FUNC(layout_image_full_screen_regrab_cb), lw);
+		g_signal_connect(G_OBJECT(menu), "selection_done",
+				 G_CALLBACK(layout_image_full_screen_regrab_cb), lw);
 		}
 }
 
-static void layout_image_button4_cb(ImageWindow *imd, GdkEventButton *bevent, gpointer data)
+static void layout_image_button4_cb(ImageWindow *imd, guint32 time, gdouble x, gdouble y, guint state, gpointer data)
 {
 	LayoutWindow *lw = data;
 
-	if (bevent->state & GDK_CONTROL_MASK)
+	if (state & GDK_CONTROL_MASK)
 		{
-		image_zoom_adjust_at_point(imd, get_zoom_increment(), bevent->x, bevent->y);
+		image_zoom_adjust_at_point(imd, get_zoom_increment(), x, y);
 		}
-	else if ( (bevent->state & GDK_SHIFT_MASK) != (mousewheel_scrolls))
+	else if ( (state & GDK_SHIFT_MASK) != (mousewheel_scrolls))
 		{
 		image_scroll(imd, 0, -MOUSEWHEEL_SCROLL_SIZE);
 		}
@@ -1180,15 +1256,15 @@ static void layout_image_button4_cb(ImageWindow *imd, GdkEventButton *bevent, gp
 		}
 }
 
-static void layout_image_button5_cb(ImageWindow *imd, GdkEventButton *bevent, gpointer data)
+static void layout_image_button5_cb(ImageWindow *imd, guint32 time, gdouble x, gdouble y, guint state, gpointer data)
 {
 	LayoutWindow *lw = data;
 
-	if (bevent->state & GDK_CONTROL_MASK)
+	if (state & GDK_CONTROL_MASK)
 		{
-		image_zoom_adjust_at_point(imd, -get_zoom_increment(), bevent->x, bevent->y);
+		image_zoom_adjust_at_point(imd, -get_zoom_increment(), x, y);
 		}
-	else if ( (bevent->state & GDK_SHIFT_MASK) != (mousewheel_scrolls))
+	else if ( (state & GDK_SHIFT_MASK) != (mousewheel_scrolls))
 		{
 		image_scroll(imd, 0, MOUSEWHEEL_SCROLL_SIZE);
 		}
@@ -1230,7 +1306,9 @@ GtkWidget *layout_image_new(LayoutWindow *lw, const gchar *path)
 		layout_image_set_buttons(lw);
 		layout_image_dnd_init(lw);
 
-		image_attach_window(lw->image, lw->window, NULL, " - GQview", FALSE);
+		image_attach_window(lw->image, lw->window, NULL, "GQview", FALSE);
+
+		image_auto_refresh(lw->image, 0);
 		}
 
 	return lw->image->widget;
