@@ -1,6 +1,6 @@
 /*
  * GQview
- * (C) 2002 John Ellis
+ * (C) 2004 John Ellis
  *
  * Author: John Ellis
  *
@@ -28,7 +28,7 @@ void layout_recent_add_path(const gchar *path);
 
 GtkWidget *layout_menu_bar(LayoutWindow *lw);
 
-GtkWidget *layout_button(GtkWidget *box, gchar **pixmap_data, gint toggle,
+GtkWidget *layout_button(GtkWidget *box, gchar **pixmap_data, const gchar *stock_id, gint toggle,
 			 GtkTooltips *tooltips, const gchar *tip_text,
 			 GtkSignalFunc func, gpointer data);
 GtkWidget *layout_button_bar(LayoutWindow *lw);
@@ -40,4 +40,17 @@ PixmapFolders *folder_icons_new(void);
 void folder_icons_free(PixmapFolders *pf);
 
 
+void layout_bar_exif_new(LayoutWindow *lw);
+void layout_bar_exif_close(LayoutWindow *lw);
+void layout_bar_exif_toggle(LayoutWindow *lw);
+
+void layout_bar_sort_new(LayoutWindow *lw);
+void layout_bar_sort_close(LayoutWindow *lw);
+void layout_bar_sort_toggle(LayoutWindow *lw);
+
+void layout_bars_new_image(LayoutWindow *lw);
+GtkWidget *layout_bars_prepare(LayoutWindow *lw, GtkWidget *image);
+void layout_bars_close(LayoutWindow *lw);
+
+
 #endif
