@@ -791,7 +791,7 @@ static void safe_delete_clear_cb(GtkWidget* widget, gpointer data)
 				"GQview", "clear_trash", TRUE, dummy_cancel_cb, NULL);
 	generic_dialog_add_stock(gd, _("Ok"), GTK_STOCK_OK, safe_delete_clear_ok_cb, TRUE);
 	entry = gtk_entry_new();
-	gtk_widget_set_sensitive(entry, FALSE);
+	gtk_editable_set_editable(GTK_EDITABLE(entry), FALSE);
 	if (safe_delete_path) gtk_entry_set_text(GTK_ENTRY(entry), safe_delete_path);
 	gtk_box_pack_start(GTK_BOX(gd->vbox), entry, FALSE, FALSE, 0);
 	gtk_widget_show(entry);
@@ -822,6 +822,7 @@ static void config_window_create(void)
 	gint i;
 
 	configwindow = gtk_window_new (GTK_WINDOW_TOPLEVEL);
+	gtk_window_set_type_hint(GTK_WINDOW(configwindow), GDK_WINDOW_TYPE_HINT_DIALOG);
 	g_signal_connect(G_OBJECT (configwindow), "delete_event",
 			 G_CALLBACK(config_window_delete), NULL);
 	gtk_window_set_resizable(GTK_WINDOW(configwindow), FALSE);
@@ -1115,6 +1116,7 @@ static void config_window_create(void)
 	gtk_widget_show(vbox1);
 
 	scrolled = gtk_scrolled_window_new(NULL, NULL);
+	gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(scrolled), GTK_SHADOW_IN);
 	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW (scrolled), GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);
 	gtk_box_pack_start(GTK_BOX(vbox1), scrolled, TRUE, TRUE, 0);
 	gtk_widget_show(scrolled);
@@ -1396,6 +1398,7 @@ void show_about_window(void)
 		}
 
 	about = gtk_window_new(GTK_WINDOW_TOPLEVEL);
+	gtk_window_set_type_hint(GTK_WINDOW(about), GDK_WINDOW_TYPE_HINT_DIALOG);
 	gtk_window_set_wmclass(GTK_WINDOW(about), "about", "GQview");
 	g_signal_connect(G_OBJECT(about), "delete_event",
 			 G_CALLBACK(about_delete_cb), NULL);
