@@ -1,6 +1,6 @@
 /*
  * GQview
- * (C) 2002 John Ellis
+ * (C) 2004 John Ellis
  *
  * Author: John Ellis
  *
@@ -47,13 +47,15 @@ static gint thumb_loader_save_to_cache(ThumbLoader *tl)
 	if (cache_ensure_dir_exists(cache_dir, mode))
 		{
 		gchar *cache_path;
+		gchar *pathl;
 
 		cache_path = g_strconcat(cache_dir, "/", filename_from_path(tl->path),
 					 GQVIEW_CACHE_THUMB_EXT, NULL);
 
 		if (debug) printf("Saving thumb: %s\n", cache_path);
 
-		success = pixbuf_to_file_as_png(tl->pixbuf, cache_path);
+		pathl = path_from_utf8(cache_path);
+		success = pixbuf_to_file_as_png(tl->pixbuf, pathl);
 		if (success)
 			{
 			struct utimbuf ut;
@@ -62,10 +64,15 @@ static gint thumb_loader_save_to_cache(ThumbLoader *tl)
 			ut.actime = ut.modtime = filetime(tl->path);
 			if (ut.modtime > 0)
 				{
-				utime(cache_path, &ut);
+				utime(pathl, &ut);
 				}
 			}
+		else
+			{
+			if (debug) printf("Saving failed: %s\n", pathl);
+			}
 
+		g_free(pathl);
 		g_free(cache_path);
 		}
 
@@ -74,6 +81,52 @@ static gint thumb_loader_save_to_cache(ThumbLoader *tl)
 	return success;
 }
 
+static gint thumb_loader_mark_failure(ThumbLoader *tl)
+{
+	gchar *cache_dir;
+	gint success = FALSE;
+	mode_t mode = 0755;
+	
+	if (!tl) return FALSE;
+
+	cache_dir = cache_get_location(tl->path, FALSE, NULL, &mode);
+
+	if (cache_ensure_dir_exists(cache_dir, mode))
+		{
+		gchar *cache_path;
+		gchar *pathl;
+		FILE *f;
+
+		cache_path = g_strconcat(cache_dir, "/", filename_from_path(tl->path),
+					 GQVIEW_CACHE_THUMB_EXT, NULL);
+
+		if (debug) printf("marking thumb failure: %s\n", cache_path);
+
+		pathl = path_from_utf8(cache_path);
+		f = fopen(pathl, "w");
+		if (f)
+			{
+			struct utimbuf ut;
+
+			fclose (f);
+
+			ut.actime = ut.modtime = filetime(tl->path);
+			if (ut.modtime > 0)
+				{
+				utime(pathl, &ut);
+				}
+
+			success = TRUE;
+			}
+
+		g_free(pathl);
+		g_free(cache_path);
+		}
+
+	g_free(cache_dir);
+	return success;
+}
+
 static void thumb_loader_percent_cb(ImageLoader *il, gfloat percent, gpointer data)
 {
 	ThumbLoader *tl = data;
@@ -118,8 +171,6 @@ static void thumb_loader_done_cb(ImageLoader *il, gpointer data)
 		return;
 		}
 
-	gdk_pixbuf_ref(pixbuf);
-
 	/* scale ?? */
 
 	if (pw > tl->max_w || ph > tl->max_h)
@@ -145,11 +196,9 @@ static void thumb_loader_done_cb(ImageLoader *il, gpointer data)
 	else
 		{
 		tl->pixbuf = pixbuf;
+		gdk_pixbuf_ref(tl->pixbuf);
 		save = FALSE;
 		}
-	gdk_pixbuf_ref(tl->pixbuf);
-
-	gdk_pixbuf_unref(pixbuf);
 
 	/* save it ? */
 	if (enable_thumb_caching && save)
@@ -242,6 +291,15 @@ gint thumb_loader_start(ThumbLoader *tl, void (*func_done)(ThumbLoader *, gpoint
 			if (filetime(cache_path) == filetime(tl->path))
 				{
 				if (debug) printf("Found in cache:%s\n", tl->path);
+
+				if (filesize(cache_path) == 0)
+					{
+					if (debug) printf("Broken image mark found:%s\n", cache_path);
+					g_free(cache_path);
+					return FALSE;
+					}
+
+				if (debug) printf("Cache location:%s\n", cache_path);
 				}
 			else
 				{
@@ -278,11 +336,17 @@ gint thumb_loader_start(ThumbLoader *tl, void (*func_done)(ThumbLoader *, gpoint
 		if (tl->from_cache)
 			{
 			tl->from_cache = FALSE;
-			printf(_("Thumbnail image in cache failed to load, trying to recreate.\n"));
+			print_term(_("Thumbnail image in cache failed to load, trying to recreate.\n"));
 			image_loader_free(tl->il);
 			thumb_loader_setup(tl, tl->path);
 			if (image_loader_start(tl->il, thumb_loader_done_cb, tl)) return TRUE;
 			}
+		/* mark failed thumbnail in cache with 0 byte file */
+		if (enable_thumb_caching)
+			{
+			thumb_loader_mark_failure(tl);
+			}
+		
 		image_loader_free(tl->il);
 		tl->il = NULL;
 		return FALSE;
@@ -300,6 +364,40 @@ gint thumb_loader_to_pixmap(ThumbLoader *tl, GdkPixmap **pixmap, GdkBitmap **mas
 	return thumb_loader_get_space(tl);
 }
 
+GdkPixbuf *thumb_loader_get_pixbuf(ThumbLoader *tl, gint with_fallback)
+{
+	GdkPixbuf *pixbuf;
+
+	if (tl && tl->pixbuf)
+		{
+		pixbuf = tl->pixbuf;
+		g_object_ref(pixbuf);
+		}
+	else if (with_fallback)
+		{
+		gint w, h;
+
+		pixbuf = gdk_pixbuf_new_from_xpm_data(img_unknown);
+		w = gdk_pixbuf_get_width(pixbuf);
+		h = gdk_pixbuf_get_height(pixbuf);
+		if ((w > tl->max_w || h > tl->max_h) &&
+		    normalize_thumb(&w, &h, tl->max_w, tl->max_h))
+			{
+			GdkPixbuf *tmp;
+
+			tmp = pixbuf;
+			pixbuf = gdk_pixbuf_scale_simple(tmp, w, h, GDK_INTERP_NEAREST);
+			gdk_pixbuf_unref(tmp);
+			}
+		}
+	else
+		{
+		pixbuf = NULL;
+		}
+
+	return pixbuf;
+}
+
 gint thumb_loader_get_space(ThumbLoader *tl)
 {
 	if (!tl) return 0;
@@ -351,7 +449,8 @@ gint thumb_from_xpm_d(const char **data, gint max_w, gint max_h, GdkPixmap **pix
 	w = gdk_pixbuf_get_width(pixbuf);
 	h = gdk_pixbuf_get_height(pixbuf);
 
-	if (normalize_thumb(&w, &h, max_w, max_h))
+	if ((w > max_w || h > max_h) &&
+	    normalize_thumb(&w, &h, max_w, max_h))
 		{
 		/* scale */
 		GdkPixbuf *tmp;
@@ -450,7 +549,7 @@ static GdkPixbuf *get_xv_thumbnail(gchar *thumb_filename, gint max_w, gint max_h
 	gchar *last_slash;
 	guchar *packed_data;
 
-	tmp_string = g_strdup(thumb_filename);
+	tmp_string = path_from_utf8(thumb_filename);
 	last_slash = strrchr(tmp_string, '/');
 	if(!last_slash)	return NULL;
 	*last_slash++ = '\0';
