@@ -1,6 +1,6 @@
 /*
  * GQview
- * (C) 2002 John Ellis
+ * (C) 2004 John Ellis
  *
  * Author: John Ellis
  *
@@ -121,11 +121,11 @@ struct _ThumbLoader
 struct _CollectInfo
 {
 	gchar *path;
-	gint size;
+	gint64 size;
 	time_t date;
 
-	GdkPixmap *pixmap;
-	GdkBitmap *mask;
+	GdkPixbuf *pixbuf;
+	gint flag_mask;
 };
 
 struct _CollectionData
@@ -157,7 +157,7 @@ struct _CollectionData
 struct _CollectTable
 {
 	GtkWidget *scrolled;
-	GtkWidget *clist;
+	GtkWidget *listview;
 	gint columns;
 	gint rows;
 
@@ -166,17 +166,13 @@ struct _CollectTable
 	GList *selection;
 	CollectInfo *prev_selection;
 
-	gint mouse_in_drag;
-	gint mouse_add;
-	CollectInfo *mouse_start;
-	CollectInfo *mouse_current;
-	gint scroll_timeout_id;
+	CollectInfo *click_info;
 
 	GtkWidget *tip_window;
 	gint tip_delay_id;
 	CollectInfo *tip_info;
 
-	GtkWidget *marker_window;
+	GdkWindow *marker_window;
 	CollectInfo *marker_info;
 
 	GtkWidget *status_label;
@@ -186,7 +182,14 @@ struct _CollectTable
 	gint focus_column;
 	CollectInfo *focus_info;
 
-	gint frozen;
+	GtkWidget *popup;
+	CollectInfo *drop_info;
+	GList *drop_list;
+
+	gint sync_idle_id;
+	gint drop_idle_id;
+
+	gint show_text;
 };
 
 struct _CollectWindow
@@ -213,7 +216,8 @@ struct _ImageWindow
 	gint image_width;	/* image actual dimensions (pixels) */
 	gint image_height;
 	GdkPixbuf *pixbuf;
-	gint size;		/* file size (bytes) */
+	gint64 size;		/* file size (bytes) */
+	time_t mtime;		/* file modified time stamp */
 	gint unknown;		/* failed to load image */
 
 	gint window_width;	/* allocated size of window (drawing area) */
@@ -263,15 +267,20 @@ struct _ImageWindow
 	gchar *title_right;	/* window title to display right of file name */
 	gint title_show_zoom;	/* option to include zoom in window title */
 
-	void (*update_func)(ImageWindow *, gpointer);
-	gpointer update_data;
+	gint completed;
+
+	void (*func_update)(ImageWindow *, gpointer);
+	void (*func_complete)(ImageWindow *, gint preload, gpointer);
+
+	gpointer data_update;
+	gpointer data_complete;
 
 	/* button functions */
-	void (*func_btn1)(ImageWindow *, GdkEventButton *, gpointer);
-	void (*func_btn2)(ImageWindow *, GdkEventButton *, gpointer);
-	void (*func_btn3)(ImageWindow *, GdkEventButton *, gpointer);
-	void (*func_btn4)(ImageWindow *, GdkEventButton *, gpointer);
-	void (*func_btn5)(ImageWindow *, GdkEventButton *, gpointer);
+	void (*func_btn1)(ImageWindow *, guint32 time, gdouble x, gdouble y, guint state, gpointer);
+	void (*func_btn2)(ImageWindow *, guint32 time, gdouble x, gdouble y, guint state, gpointer);
+	void (*func_btn3)(ImageWindow *, guint32 time, gdouble x, gdouble y, guint state, gpointer);
+	void (*func_btn4)(ImageWindow *, guint32 time, gdouble x, gdouble y, guint state, gpointer);
+	void (*func_btn5)(ImageWindow *, guint32 time, gdouble x, gdouble y, guint state, gpointer);
 
 	gpointer data_btn1;
 	gpointer data_btn2;
@@ -288,13 +297,21 @@ struct _ImageWindow
 	ImageLoader *read_ahead_il;
 	GdkPixbuf *read_ahead_pixbuf;
 	gchar *read_ahead_path;
+
+	GdkPixbuf *prev_pixbuf;
+	gchar *prev_path;
+
+	gint auto_refresh_id;
+	gint auto_refresh_interval;
 };
 
 struct _FileData {
 	gchar *path;
 	const gchar *name;
-	gint size;
+	gint64 size;
 	time_t date;
+
+	GdkPixbuf *pixbuf;
 };
 
 struct _LayoutWindow
@@ -391,19 +408,31 @@ struct _LayoutWindow
 
 	gint last_time_id;
 	time_t last_time;
+
+	/* misc */
+
+	GtkWidget *utility_box;
+	GtkWidget *bar_sort;
+	GtkWidget *bar_exif;
+
+	gint bar_sort_enabled;
+	gint bar_exif_enabled;
+
+	gint bar_exif_size;
+	gint bar_exif_advanced;
 };
 
 struct _ViewDirList
 {
 	GtkWidget *widget;
-	GtkWidget *clist;
+	GtkWidget *listview;
 
 	gchar *path;
 	GList *list;
 
-	gint click_row;
+	FileData *click_fd;
 
-	gint drop_row;
+	FileData *drop_fd;
 	GList *drop_list;
 
 	gint drop_scroll_id;
@@ -422,18 +451,17 @@ struct _ViewDirList
 struct _ViewDirTree
 {
 	GtkWidget *widget;
-	GtkWidget *ctree;
+	GtkWidget *treeview;
 
 	gchar *path;
 
-	GtkCTreeNode *click_node;
+	FileData *click_fd;
 
-	GtkCTreeNode *drop_node;
+	FileData *drop_fd;
 	GList *drop_list;
 
 	gint drop_scroll_id;
 	gint drop_expand_id;
-	gint drop_row;
 
 	/* func list */
 	void (*select_func)(ViewDirTree *vdt, const gchar *path, gpointer data);
@@ -444,12 +472,14 @@ struct _ViewDirTree
 	GtkWidget *popup;
 
 	PixmapFolders *pf;
+
+	gint busy_ref;
 };
 
 struct _ViewFileList
 {
 	GtkWidget *widget;
-	GtkWidget *clist;
+	GtkWidget *listview;
 
 	gchar *path;
 	GList *list;
@@ -457,15 +487,13 @@ struct _ViewFileList
 	SortType sort_method;
 	gint sort_ascend;
 
-	gint click_row;
-	gint click_time;
-	gint rename_row;
+	FileData *click_fd;
+	FileData *select_fd;
 
 	gint thumbs_enabled;
 
 	/* thumb updates */
 	gint thumbs_running;
-	GList *thumbs_list;
 	gint thumbs_count;
 	ThumbLoader *thumbs_loader;
 	FileData *thumbs_filedata;
@@ -486,7 +514,7 @@ struct _ViewFileList
 struct _ViewFileIcon
 {
 	GtkWidget *widget;
-	GtkWidget *clist;
+	GtkWidget *listview;
 
 	gchar *path;
 	GList *list;
@@ -499,26 +527,20 @@ struct _ViewFileIcon
 	GList *selection;
 	FileData *prev_selection;
 
-	gint mouse_in_drag;
-	gint mouse_add;
-	FileData *mouse_start;
-	FileData *mouse_current;
-	gint scroll_timeout_id;
-
 	GtkWidget *tip_window;
 	gint tip_delay_id;
 	FileData *tip_fd;
 
+	FileData *click_fd;
+
+	FileData *focus_fd;
 	gint focus_row;
 	gint focus_column;
-	FileData *focus_fd;
-
-	gint frozen;
 
 	SortType sort_method;
 	gint sort_ascend;
 
-	FileData *click_fd;
+	gint show_text;
 
 	gint sync_idle_id;
 
@@ -581,15 +603,11 @@ struct _FullScreenData
 	gpointer stop_data;
 };
 
-struct _PixmapFolders {
-	GdkPixmap *close_pixmap;
-	GdkBitmap *close_mask;
-
-	GdkPixmap *open_pixmap;
-	GdkBitmap *open_mask;
-
-	GdkPixmap *deny_pixmap;
-	GdkBitmap *deny_mask;
+struct _PixmapFolders
+{
+	GdkPixbuf *close;
+	GdkPixbuf *open;
+	GdkPixbuf *deny;
 };
 
 
