@@ -1201,7 +1201,7 @@ static void file_util_delete_multiple_ok_cb(GenericDialog *gd, gpointer data)
 				g_free(text);
 
 				generic_dialog_add_stock(d, _("Continue"), GTK_STOCK_GO_FORWARD, file_util_delete_multiple_ok_cb, TRUE);
-				gtk_widget_show(gd->dialog);
+				gtk_widget_show(d->dialog);
 				}
 			else
 				{
@@ -1487,6 +1487,8 @@ struct _RenameDataMult
 	GtkWidget *auto_entry_end;
 
 	ImageWindow *imd;
+
+	gint update_idle_id;
 };
 
 static void file_util_rename_multiple(RenameDataMult *rd);
@@ -1818,16 +1820,24 @@ static void file_util_rename_multiple_preview_update(RenameDataMult *rd)
 
 }
 
-static void file_util_rename_multiple_preview_order_cb(GtkTreeModel *treemodel, GtkTreePath *tpath, GtkTreeIter *iter,
-						       gint *new_order, gpointer data)
+static gboolean file_util_rename_multiple_idle_cb(gpointer data)
 {
 	RenameDataMult *rd = data;
-	GtkTreeViewColumn *column;
 
-	column = gtk_tree_view_get_column(GTK_TREE_VIEW(rd->listview), RENAME_COLUMN_PREVIEW);
-	if (column && gtk_tree_view_column_get_visible(column))
+	file_util_rename_multiple_preview_update(rd);
+
+	rd->update_idle_id = -1;
+	return FALSE;
+}
+
+static void file_util_rename_multiple_preview_order_cb(GtkTreeModel *treemodel, GtkTreePath *tpath,
+						       GtkTreeIter *iter, gpointer data)
+{
+	RenameDataMult *rd = data;
+
+	if (rd->rename_auto && rd->update_idle_id == -1)
 		{
-		file_util_rename_multiple_preview_update(rd);
+		rd->update_idle_id = gtk_idle_add(file_util_rename_multiple_idle_cb, rd);
 		}
 }
 
@@ -1854,6 +1864,7 @@ static void file_util_rename_mulitple_auto_toggle(GtkWidget *widget, gpointer da
 		{
 		if (GTK_WIDGET_VISIBLE(rd->rename_box)) gtk_widget_hide(rd->rename_box);
 		if (!GTK_WIDGET_VISIBLE(rd->auto_box)) gtk_widget_show(rd->auto_box);
+		file_util_rename_multiple_preview_update(rd);
 		}
 	else
 		{
@@ -1918,6 +1929,8 @@ static void file_util_rename_multiple_destroy_cb(GtkWidget *widget, gpointer dat
 {
 	RenameDataMult *rd = data;
 
+	if (rd->update_idle_id != -1) gtk_idle_remove(rd->update_idle_id);
+
 	g_free(rd);
 }
 
@@ -1946,6 +1959,8 @@ static void file_util_rename_multiple_do(GList *source_list)
 	rd->fd->source_path = g_strdup(source_list->data);
 	rd->fd->dest_path = NULL;
 
+	rd->update_idle_id = -1;
+
 	vbox = GENERIC_DIALOG(rd->fd)->vbox;
 
 	pane = gtk_hpaned_new();
@@ -1953,6 +1968,7 @@ static void file_util_rename_multiple_do(GList *source_list)
 	gtk_widget_show(pane);
 	
 	scrolled = gtk_scrolled_window_new(NULL, NULL);
+	gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(scrolled), GTK_SHADOW_IN);
 	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW (scrolled),
 				       GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);
 	gtk_paned_pack1(GTK_PANED(pane), scrolled, TRUE, TRUE);
@@ -1971,7 +1987,7 @@ static void file_util_rename_multiple_do(GList *source_list)
 	gtk_tree_view_column_set_visible(column, FALSE);
 	
 	gtk_tree_view_set_reorderable(GTK_TREE_VIEW(rd->listview), TRUE);
-	g_signal_connect(G_OBJECT(store), "rows_reordered",
+	g_signal_connect(G_OBJECT(store), "row_changed",
 			 G_CALLBACK(file_util_rename_multiple_preview_order_cb), rd);
 	gtk_widget_set_size_request(rd->listview, 250, 150);
 
