@@ -241,7 +241,7 @@ static void vdtree_rename_by_data(ViewDirTree *vdt, FileData *fd)
 static void vdtree_node_free(NodeData *nd)
 {
 	if (!nd) return;
-                                                                                                                               
+
 	file_data_free(nd->fd);
 	g_free(nd);
 }
@@ -1001,7 +1001,7 @@ static void vdtree_add_by_data(ViewDirTree *vdt, FileData *fd, GtkTreeIter *pare
 	 * this is removed when the child is populated */
 	end = g_new0(NodeData, 1);
 	end->fd = g_new0(FileData, 1);
-	end->fd->path = g_strdup("empty");
+	end->fd->path = g_strdup("");
 	end->fd->name = end->fd->path;
 	end->expanded = TRUE;
 
@@ -1125,12 +1125,12 @@ static gint vdtree_populate_path_by_iter(ViewDirTree *vdt, GtkTreeIter *iter, gi
 			if (cnd)
 				{
 				old = g_list_remove(old, cnd);
-				if (cnd->expanded && cnd->fd->date != fd->date)
+				if (cnd->expanded && cnd->fd->date != fd->date &&
+				    vdtree_populate_path_by_iter(vdt, &child, FALSE, target_path))
 					{
-					vdtree_populate_path_by_iter(vdt, &child, FALSE, target_path);
+					cnd->fd->size = fd->size;
+					cnd->fd->date = fd->date;
 					}
-				cnd->fd->size = fd->size;
-				cnd->fd->date = fd->date;
 
 				file_data_free(fd);
 				}
@@ -1671,6 +1671,7 @@ ViewDirTree *vdtree_new(const gchar *path, gint expand)
 	vdt->busy_ref = 0;
 
 	vdt->widget = gtk_scrolled_window_new(NULL, NULL);
+	gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(vdt->widget), GTK_SHADOW_IN);
 	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(vdt->widget),
 				       GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);
 	g_signal_connect(G_OBJECT(vdt->widget), "destroy",
